package dbpatient;

import java.io.Console;
import java.io.FileOutputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Vector;
import java.util.Scanner;
import java.util.Date;
import java.io.Serializable;
import nu.xom.Attribute;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Serializer;
import java.io.ObjectOutputStream;

public class OraclePatient implements Serializable
{
  // Do we get all of the data or only what is necessary for prediction system
  public static boolean All = false;
  // Do we load basis data?
  public static boolean Basis = false;
  // Do we Filter/average Basis data
  public static boolean Filtered = false;

  // Patient ID.
  public int id_;
  public int weight_; // in pounds.
  
  // Metadata for Insulin and Carbs.
  public String insulin_type_;
  Vector<TSEInsulinSensitivity> insulin_sensitivity_;
  Vector<TSECarbRatio> carb_ratio_;
  
  // Blood Glucose measurements.
  Vector<TSEGlucoseLevel> glucose_level_;
  Vector<TSEFingerStick> finger_stick_;
  
  // Insulin events.
  Vector<TSEBasal> basal_;
  Vector<TSETempBasal> temp_basal_;
  Vector<TSEBolus> bolus_;
  
  // Meal events.
  Vector<TSEMeal> meal_;
  
  // Other life events.
  Vector<TSESleep> sleep_;
  Vector<TSEWork> work_;
  
  // Insulin pump events.
  Vector<TSEInfusionSet> infusion_set_;
  Vector<TSEPumpFailure> pump_failure_;

  // Loaded and output only if --all is specified in command line
  Vector<TSEStressors> stressors_;
  Vector<TSEMenses> menses_;
  Vector<TSEHypoEvent> hypo_event_;
  Vector<TSEHypoAction> hypo_action_;
  Vector<TSEMiscEvent> misc_event_;
  Vector<TSEIllness> illness_;
  Vector<TSEExercise> exercise_;
  
  // Loaded and output only if --basis is specified in command line
  Vector<TSEBasisSleep> basis_sleep_;
  Vector<TSEBasisAction> basis_action_;
  Vector<TSEBasisHeartRate> basis_heart_rate_;
  Vector<TSEBasisSteps> basis_steps_;
  Vector<TSEBasisCalories> basis_calories_;
  Vector<TSEBasisGSR> basis_gsr_;
  Vector<TSEBasisTemp> basis_skin_temp_;
  Vector<TSEBasisTemp> basis_air_temp_;
  Vector<TSEBasisActivity>basis_activity_;

  public OraclePatient(int id)
  {
    id_ = id;
  }
  
  public Connection getConnection() throws SQLException
  {
    String db_url = "jdbc:oracle:oci:@";
    Console cons = System.console();
    System.out.println("Username:");
    Scanner scanner = new Scanner(System.in);
    String db_user = scanner.nextLine();
    String db_passwd;
    if (cons != null){
      db_passwd = new String(cons.readPassword("%s", "Password:"));
    }
    else {
      System.out.println("Password:");
      db_passwd = scanner.nextLine();
    }
    try {
      Class.forName("oracle.jdbc.driver.OracleDriver");
    }
    catch (ClassNotFoundException e) {
      System.out.println("Coud not load the driver");
    }
    return DriverManager.getConnection(db_url, db_user, db_passwd); // return connection;
  }
  
  public void loadFromDB(Connection conn) throws SQLException
  {
    loadInsulinType(conn);
    loadWeight(conn);
    loadInsulinSensitivity(conn);
    loadCarbRatio(conn);
    loadGlucoseLevel(conn);
    loadFingerStick(conn);
    loadBasal(conn);
    loadTempBasal(conn);
    loadBolus(conn);
    loadMeal(conn);
    loadSleep(conn);
    loadWork(conn);
    loadInfusionSet(conn);
    loadPumpFailure(conn);

    if (All){
      loadStressors(conn);
      loadMenses(conn);
      loadHypoEvent(conn);
      loadHypoAction(conn);
      loadMiscEvent(conn);
      loadIllness(conn);
      loadExercise(conn);
    }
    if (Basis){
      loadBasisHeartRate(conn);
      loadBasisGSR(conn);
      loadBasisSkinTemp(conn);
      loadBasisAirTemp(conn);

      loadBasisSleep(conn);
      loadBasisAction(conn);
      loadBasisSteps(conn);
      loadBasisCalories(conn);
      loadBasisActivity(conn);
    }
  }
  
  
  private void loadInsulinType(Connection conn) throws SQLException
  {
    String query = "Select type From Insulin_Type Where pid = " + id_; 
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (!rset.next()){ // The most likely cause of this failing is due to patient not existing
      System.err.println("Warning: No patient found with id = " + id_ + " Now Exiting Program");
      System.exit(-1);
    }
    insulin_type_ = rset.getString("type");
    if (rset.next())
      System.err.println("Warning: multiple insulin_type entries for patient " + id_ + "!");
    rset.close();
    stmt.close();
  }

  
  private void loadWeight(Connection conn) throws SQLException
  {
    String query = "Select lbs From Weight Where pid = " + id_; 
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    rset.next();
    weight_ = rset.getInt("lbs");
    if (rset.next())
      System.err.println("Warning: multiple Weight entries for patient " + id_ + "!");
    rset.close();
    stmt.close();
  }
  

  private void loadInsulinSensitivity(Connection conn) throws SQLException
  {
    insulin_sensitivity_ = new Vector<TSEInsulinSensitivity>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp, val From Insulin_Sensitivity Where pid = " 
                    + id_ + " Order By ts"; 
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEInsulinSensitivity is = new TSEInsulinSensitivity();
      is.setTS(rset.getString("tstamp"));
      is.value_ = rset.getDouble("val");
      insulin_sensitivity_.add(is);
    }
    rset.close();
    stmt.close();
  }


  private void loadCarbRatio(Connection conn) throws SQLException
  {
    carb_ratio_ = new Vector<TSECarbRatio>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp, ratio From Carb_Ratio Where pid = " 
                    + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSECarbRatio cr = new TSECarbRatio();
      cr.setTS(rset.getString("tstamp"));
      cr.value_ = rset.getInt("ratio");
      carb_ratio_.add(cr);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadGlucoseLevel(Connection conn) throws SQLException
  {
    glucose_level_ = new Vector<TSEGlucoseLevel>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp, val From Cgms Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEGlucoseLevel gl = new TSEGlucoseLevel();
      gl.setTS(rset.getString("tstamp"));
      gl.value_ = rset.getInt("val");
      glucose_level_.add(gl);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadFingerStick(Connection conn) throws SQLException
  {
    finger_stick_ = new Vector<TSEFingerStick>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp, val From Finger_Stick Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEFingerStick fs = new TSEFingerStick();
      fs.setTS(rset.getString("tstamp"));
      fs.value_ = rset.getInt("val");
      finger_stick_.add(fs);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadBasal(Connection conn) throws SQLException
  {
    basal_ = new Vector<TSEBasal>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp, val From Basal Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEBasal br = new TSEBasal();
      br.setTS(rset.getString("tstamp"));
      br.value_ = rset.getDouble("val");
      basal_.add(br);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadTempBasal(Connection conn) throws SQLException
  {
    temp_basal_ = new Vector<TSETempBasal>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, to_char(ts_end, 'dd-mm-yyyy hh24:mi:ss') As tend, val From Temp_Basal Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSETempBasal tb = new TSETempBasal();
      tb.setTS(rset.getString("tbegin"));
      tb.setEndTS(rset.getString("tend"));
      tb.value_ = rset.getDouble("val");
      temp_basal_.add(tb);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadBolus(Connection conn) throws SQLException
  {
    bolus_ = new Vector<TSEBolus>();

    String query;
    if (id_ < 400 && id_ != 101){
      query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, type, dose From Bolus Where pid = "
                    + id_ + " Order By ts";
    } else {
      query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, to_char(ts_end, 'dd-mm-yyyy hh24:mi:ss') As tend, type, dose From Bolus Where pid = "
                    + id_ + " Order By ts";
    }
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEBolus eb = new TSEBolus();
      eb.setTS(rset.getString("tbegin"));
      eb.setType(rset.getString("type"));
      if (eb.btype_ == TSEBolus.BolusType.NORMAL || eb.btype_ == TSEBolus.BolusType.NORMAL_DUAL || (id_ < 400 && id_ != 101)) // Normal bolus has 0 duration (anyone with an id_ below 300 (except 101) does not have a stored end_bolus)
        eb.end_bolus_ = eb.ts_;
      else
        eb.setEndTS(rset.getString("tend"));
      eb.dose_ = rset.getDouble("dose");
      bolus_.add(eb);
    }
    rset.close();
    stmt.close();
  }


  private void loadMeal(Connection conn) throws SQLException
  {
    meal_ = new Vector<TSEMeal>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp, name, est_carbs From Meal Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEMeal meal = new TSEMeal();
      meal.setTS(rset.getString("tstamp"));
      meal.type_ = rset.getString("name");
      meal.carbs_ = rset.getInt("est_carbs");
      // Add food eaten
      if (All){
        String query1 = "Select food_name, amount, units, alcoholic from food where m_pid="
            + id_ + " and m_ts=to_date('" + rset.getString("tstamp") + "', 'dd-mm-yyyy hh24:mi:ss')";
        Statement stmt1 = conn.createStatement();
        ResultSet rset1 = stmt1.executeQuery(query1);
        while (rset1.next()){
         meal.add_food(rset1.getString("food_name"), rset1.getDouble("amount"),
                       rset1.getString("units"), rset1.getString("alcoholic"));
        }
      }
      meal_.add(meal);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadSleep(Connection conn) throws SQLException
  {
     sleep_ = new Vector<TSESleep>();
    String query;
	//Sleep is different for 400 level patients and after (101 is similar to 500 level patients)
	//sleep now contains "quality" field
	if(id_ < 400 && id_ != 101)
	{
		query = "Select to_char(awake, 'dd-mm-yyyy hh24:mi:ss') As tbegin, to_char(to_bed, 'dd-mm-yyyy hh24:mi:ss') As tend From Sleep Where pid = " 
        + id_ + " Order By awake";
	}
	else
	{
    		query = "Select to_char(awake, 'dd-mm-yyyy hh24:mi:ss') As tbegin, to_char(to_bed, 'dd-mm-yyyy hh24:mi:ss') As tend, quality From Sleep Where pid = " 
        + id_ + " Order By awake";
	}
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSESleep sleep = new TSESleep();
      sleep.setTS(rset.getString("tbegin"));
      sleep.setEndTS(rset.getString("tend"));
	if(id_ >= 400)
	{
      		sleep.quality_ = rset.getInt("quality");
	}
      sleep_.add(sleep);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadWork(Connection conn) throws SQLException
  {
    work_ = new Vector<TSEWork>();
    
    String query = "Select to_char(to_work, 'dd-mm-yyyy hh24:mi:ss') As tbegin, to_char(from_work, 'dd-mm-yyyy hh24:mi:ss') As tend, intensity From Works Where pid = " 
        + id_ + " Order By to_work";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEWork work = new TSEWork();
      work.setTS(rset.getString("tbegin"));
      work.setEndTS(rset.getString("tend"));
      work.intensity_ = rset.getInt("intensity");
      work_.add(work);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadInfusionSet(Connection conn) throws SQLException
  {
    infusion_set_ = new Vector<TSEInfusionSet>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tstamp from Infusion_Set Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEInfusionSet is = new TSEInfusionSet();
      is.setTS(rset.getString("tstamp"));
      infusion_set_.add(is);
    }
    rset.close();
    stmt.close();
  }

  
  private void loadPumpFailure(Connection conn) throws SQLException
  {
    pump_failure_ = new Vector<TSEPumpFailure>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, to_char(stop, 'dd-mm-yyyy hh24:mi:ss') As tend, failure_type From Pump_Failure Where pid = " 
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEPumpFailure pf = new TSEPumpFailure();
      pf.setTS(rset.getString("tbegin"));
      pf.setEndTS(rset.getString("tend"));
      pf.diagnostic_ = rset.getString("failure_type");
      pump_failure_.add(pf);
    }
    rset.close();
    stmt.close();
  }

  private void loadStressors(Connection conn) throws SQLException
  {
    stressors_ = new Vector<TSEStressors>();

    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, type, description from stressor where pid="
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEStressors st = new TSEStressors();
      st.setTS(rset.getString("tbegin"));
      st.type_ = rset.getString("type");
      st.description_ = rset.getString("description");
      stressors_.add(st);
    }
    rset.close();
    stmt.close();
  }


  private void loadMenses(Connection conn) throws SQLException
  {
    menses_ = new Vector<TSEMenses>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, description, symptom_severity from menses where pid = "
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEMenses men = new TSEMenses();
      men.setTS(rset.getString("tbegin"));
      men.setDescription(rset.getString("description"));
      men.severity_ = rset.getInt("symptom_severity");
      menses_.add(men);
    }
    rset.close();
    stmt.close();
  }


  private void loadHypoEvent(Connection conn) throws SQLException
  {
    hypo_event_ = new Vector<TSEHypoEvent>();

    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin from hypo_event where pid = "
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEHypoEvent he = new TSEHypoEvent();
      he.setTS(rset.getString("tbegin"));
      query = "Select name from he_symptoms where he_pid = " + id_ + " and he_ts = to_date('" +
              rset.getString("tbegin") + "', 'dd-mm-yyyy hh24:mi:ss')";
      Statement stmt2 = conn.createStatement();
      ResultSet rset2 = stmt2.executeQuery(query);
      while (rset2.next()){
        he.addSymptom(rset2.getString("name"));
      }
      rset2.close();
      stmt2.close();
      hypo_event_.add(he);
    }
    rset.close();
    stmt.close();
  }


  private void loadHypoAction(Connection conn) throws SQLException
  {
    hypo_action_ = new Vector<TSEHypoAction>();

    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, action, est_carbs from he_action where he_pid = "
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEHypoAction ha = new TSEHypoAction();
      ha.setTS(rset.getString("tbegin"));
      ha.action_ = rset.getString("action");
      ha.carbs_ = rset.getInt("est_carbs");
      hypo_action_.add(ha);
    }
    rset.close();
    stmt.close();
  }


  private void loadMiscEvent(Connection conn) throws SQLException
  {
    misc_event_ = new Vector<TSEMiscEvent>();

    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') As tbegin, description from misc_event where pid = "
        + id_ + "Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()) {
      TSEMiscEvent me = new TSEMiscEvent();
      me.setTS(rset.getString("tbegin"));
      me.description_ = rset.getString("description");
      misc_event_.add(me);
    }
    rset.close();
    stmt.close();
  }


  private void loadIllness(Connection conn) throws SQLException
  {
    illness_ = new Vector<TSEIllness>();
    
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, to_char(stop, 'dd-mm-yyyy hh24:mi:ss') as tend, type, description from illness where pid = "
        + id_ + "Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()){
      TSEIllness il = new TSEIllness();
      il.setTS(rset.getString("tbegin"));
      il.setEndTS(rset.getString("tend"));
      il.setType(rset.getString("type"));
      il.setDescription(rset.getString("description"));

      String query1 = "Select name from isymptom where s_pid = "
          + id_ + " and s_ts = to_date('" + il.getTS() + "', 'dd-mm-yyyy hh24:mi:ss')";
      Statement stmt1 = conn.createStatement();
      ResultSet rset1 = stmt1.executeQuery(query1);
      while (rset1.next()){
        il.addSymptom(rset1.getString("name"));
      }
      rset1.close();
      stmt1.close();

      illness_.add(il);
    }
    rset.close();
    stmt.close();
  }


  private void loadExercise(Connection conn) throws SQLException
  {
    exercise_ = new Vector<TSEExercise>();

    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, intensity, type, duration, competitive from exercise where pid = "
        + id_ + " Order By ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()){
      TSEExercise ex = new TSEExercise();
      ex.setTS(rset.getString("tbegin"));
      ex.intensity_ = rset.getInt("intensity");
      ex.setType(rset.getString("type"));
      ex.duration_ = rset.getInt("duration");
      ex.setCompetitive(rset.getString("competitive"));
      exercise_.add(ex);
    }
    rset.close();
    stmt.close();
  }


  private void loadBasisSleep(Connection conn) throws SQLException
  {
    basis_sleep_ = new Vector<TSEBasisSleep>();
    String query = "Select to_char(to_bed, 'dd-mm-yyyy hh24:mi:ss') as tbegin, to_char(awake, 'dd-mm-yyyy hh24:mi:ss') as tend, quality, type from basis_sleep where pid="
        + id_+ " Order by to_bed";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()){
      TSEBasisSleep bs = new TSEBasisSleep();
      bs.setTS(rset.getString("tbegin"));
      bs.setEndTS(rset.getString("tend"));
      bs.quality_ = rset.getInt("quality");
      bs.type_ = rset.getString("type");

      String query2 = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin from basis_toss_turn where S_PID = "
          + id_ +" and s_ts = to_date('"+bs.getTS()+"', 'dd-mm-yyyy hh24:mi:ss') order by ts";
      Statement stmt2 = conn.createStatement();
      ResultSet rset2 = stmt2.executeQuery(query2);
      while (rset2.next()){
        bs.addTossAndTurnEvent(rset2.getString("tbegin"));
      }
      rset2.close();
      stmt2.close();
      basis_sleep_.add(bs);
    }
    rset.close();
    stmt.close();
  }


  private void loadBasisAction(Connection conn) throws SQLException
  {
    basis_action_ = new Vector<TSEBasisAction>();
    String query = "Select to_char(begin, 'dd-mm-yyyy hh24:mi:ss') as tbegin, to_char(end, 'dd-mm-yyyy hh24:mi:ss') as tend, type, seconds, steps, calories from basis_action where pid="
        + id_ + " Order by begin";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()){
      TSEBasisAction ba = new TSEBasisAction();
      ba.setTS(rset.getString("tbegin"));
      ba.setEndTS(rset.getString("tbegin"));
      ba.type_ = rset.getString("type");
      ba.seconds_ = rset.getInt("seconds");
      ba.steps_ = rset.getInt("steps");
      ba.cal_ = rset.getDouble("calories");
      basis_action_.add(ba);
    }
    rset.close();
    stmt.close();
  }


  private void loadBasisHeartRate(Connection conn) throws SQLException
  {
    basis_heart_rate_ = new Vector<TSEBasisHeartRate>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, val from basis_heart_rate where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (Filtered){
      int weight = 1;
      TSEBasisHeartRate bhr = new TSEBasisHeartRate();
      TSEvent tmp = new TSEvent();
      Date time_next, time_two_next;
      int filter_inc = 1000*60*5;
      if (rset.next()){
        bhr.setTS(rset.getString("tbegin"));
        bhr.value_ = rset.getInt("val");
        time_next = new Date(bhr.ts_.getTime() + filter_inc); // Time + 5 minutes.
        time_two_next = new Date(time_next.getTime() + filter_inc); // Time + 10 minutes;

        while (rset.next()){
          tmp.setTS(rset.getString("tbegin"));
          if (tmp.ts_.before(time_next)){
            bhr.value_ += rset.getInt("val");
            weight++;
          } else {
            bhr.value_ /= weight;
            basis_heart_rate_.add(bhr);
            if (tmp.ts_.before(time_two_next)){
              bhr = new TSEBasisHeartRate(time_next);
              time_next = time_two_next;
            } else {
              bhr = new TSEBasisHeartRate(time_next);  // add a null value to show that there was a gap in data
              bhr.value_ = 0;
              basis_heart_rate_.add(bhr);
              bhr = new TSEBasisHeartRate(tmp.ts_);
              time_next = new Date(bhr.ts_.getTime() + filter_inc);
            }
            time_two_next = new Date(time_next.getTime() + filter_inc);
            bhr.value_ = rset.getInt("val");
            weight = 1;
          }
        }
        bhr.value_ /= weight;
        basis_heart_rate_.add(bhr);
      }
    } else {
      while (rset.next()){
        TSEBasisHeartRate bhr = new TSEBasisHeartRate();
        bhr.setTS(rset.getString("tbegin"));
        bhr.value_ = rset.getInt("val");
        basis_heart_rate_.add(bhr);
      }
    }
    rset.close();
    stmt.close();
  }


  private void loadBasisSteps(Connection conn) throws SQLException
  {
    basis_steps_ = new Vector<TSEBasisSteps>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, val from basis_steps where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (Filtered){
      TSEBasisSteps bs = new TSEBasisSteps();
      TSEvent tmp = new TSEvent();
      Date time_next, time_two_next;
      int filter_inc = 1000*60*5;
      if (rset.next()){
        bs.setTS(rset.getString("tbegin"));
        bs.value_ = rset.getInt("val");
        time_next = new Date(bs.ts_.getTime() + filter_inc);  // Time + 5 minutes.
        time_two_next = new Date(time_next.getTime() + filter_inc); // Time + 10 minutes.
        while (rset.next()){
          tmp.setTS(rset.getString("tbegin"));
          if (tmp.ts_.before(time_next)){
            bs.value_ += rset.getInt("val");
          } else {
            basis_steps_.add(bs);
            if (tmp.ts_.before(time_two_next)){
              bs = new TSEBasisSteps(time_next);
              time_next = time_two_next;
            } else {
              bs = new TSEBasisSteps(tmp.ts_);
              time_next = new Date(bs.ts_.getTime() + filter_inc);
            }
            time_two_next = new Date(time_next.getTime() + filter_inc);
            bs.value_ = rset.getInt("val");
          }
        }
        basis_steps_.add(bs);
      }
    } else {
      while (rset.next()){
        TSEBasisSteps bs = new TSEBasisSteps();
        bs.setTS(rset.getString("tbegin"));
        bs.value_ = rset.getInt("val");
        basis_steps_.add(bs);
      }
    }
    rset.close();
    stmt.close();
  }

  private void loadBasisCalories(Connection conn) throws SQLException
  {
    basis_calories_ = new Vector<TSEBasisCalories>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, val from basis_calories where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (Filtered){
      int weight = 1;
      TSEBasisCalories bc = new TSEBasisCalories();
      TSEvent tmp = new TSEvent();
      Date time_next, time_two_next;
      int filter_inc = 1000*60*5;
      if (rset.next()){
        bc.setTS(rset.getString("tbegin"));
        bc.value_ = rset.getDouble("val");
        time_next = new Date(bc.ts_.getTime() + filter_inc); // Time + 5 min
        time_two_next = new Date(time_next.getTime() + filter_inc); // Time + 10 min
        while (rset.next()){
          tmp.setTS(rset.getString("tbegin"));
          if (tmp.ts_.before(time_next)){
            bc.value_ += rset.getDouble("val");
            ++weight;
          } else {
            if (tmp.ts_.before(time_two_next)){
              bc = new TSEBasisCalories(time_next);
              time_next = time_two_next;
            } else {
              bc = new TSEBasisCalories(time_next);
              bc.value_ = 0;
              basis_calories_.add(bc);
              bc = new TSEBasisCalories(tmp.ts_);
              time_next = new Date(bc.ts_.getTime() + filter_inc);
            }
            time_two_next = new Date(time_next.getTime() + filter_inc);
            bc.value_ = rset.getDouble("val");
            weight = 1;
          }
        }
        bc.value_ /= weight;
        basis_calories_.add(bc);
      }
    } else {
      while (rset.next()){
        TSEBasisCalories bc = new TSEBasisCalories();
        bc.setTS(rset.getString("tbegin"));
        bc.value_ = rset.getDouble("val");
        basis_calories_.add(bc);
      }
    }
    rset.close();
    stmt.close();
  }

  private void loadBasisGSR(Connection conn) throws SQLException
  {
    basis_gsr_ = new Vector<TSEBasisGSR>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, val from basis_gsr where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (Filtered){
      int weight = 1;
      TSEBasisGSR bgsr = new TSEBasisGSR();
      TSEvent tmp = new TSEvent();
      Date time_next, time_two_next;
      int filter_inc = 1000*60*5;
      if (rset.next()){
        bgsr.setTS(rset.getString("tbegin"));
        bgsr.value_ = rset.getDouble("val");
        time_next = new Date(bgsr.ts_.getTime() + filter_inc); // Time + 5 minutes.
        time_two_next = new Date(time_next.getTime() + filter_inc); // Time + 10 minutes;

        while (rset.next()){
          tmp.setTS(rset.getString("tbegin"));
          if (tmp.ts_.before(time_next)){
            bgsr.value_ += rset.getDouble("val");
            weight++;
          } else {
            bgsr.value_ /= weight;
            basis_gsr_.add(bgsr);
            if (tmp.ts_.before(time_two_next)){
              bgsr = new TSEBasisGSR(time_next);
              time_next = time_two_next;
            } else {
              bgsr = new TSEBasisGSR(time_next);  // add a null value to show that there was a gap in data
              bgsr.value_ = 0;
              basis_gsr_.add(bgsr);
              bgsr = new TSEBasisGSR(tmp.ts_);
              time_next = new Date(bgsr.ts_.getTime() + filter_inc);
            }
            time_two_next = new Date(time_next.getTime() + filter_inc);
            bgsr.value_ = rset.getDouble("val");
            weight = 1;
          }
        }
        bgsr.value_ /= weight;
        basis_gsr_.add(bgsr);
      }
    } else {
      while (rset.next()){
        TSEBasisGSR bgsr = new TSEBasisGSR();
        bgsr.setTS(rset.getString("tbegin"));
        bgsr.value_ = rset.getDouble("val");
        basis_gsr_.add(bgsr);
      }
    }
    rset.close();
    stmt.close();
  }

  private void loadBasisSkinTemp(Connection conn) throws SQLException
  {
    basis_skin_temp_ = new Vector<TSEBasisTemp>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, val from basis_skin_temp where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (Filtered){
      int weight = 1;
      TSEBasisTemp bt = new TSEBasisTemp();
      TSEvent tmp = new TSEvent();
      Date time_next, time_two_next;
      int filter_inc = 1000*60*5;
      if (rset.next()){
        bt.setTS(rset.getString("tbegin"));
        bt.value_ = rset.getDouble("val");
        time_next = new Date(bt.ts_.getTime() + filter_inc); // Time + 5 minutes.
        time_two_next = new Date(time_next.getTime() + filter_inc); // Time + 10 minutes;

        while (rset.next()){
          tmp.setTS(rset.getString("tbegin"));
          if (tmp.ts_.before(time_next)){
            bt.value_ += rset.getDouble("val");
            weight++;
          } else {
            bt.value_ /= weight;
            basis_skin_temp_.add(bt);
            if (tmp.ts_.before(time_two_next)){
              bt = new TSEBasisTemp(time_next);
              time_next = time_two_next;
            } else {
              bt = new TSEBasisTemp(time_next);  // add a null value to show that there was a gap in data
              bt.value_ = 0;
              basis_skin_temp_.add(bt);
              bt = new TSEBasisTemp(tmp.ts_);
              time_next = new Date(bt.ts_.getTime() + filter_inc);
            }
            time_two_next = new Date(time_next.getTime() + filter_inc);
            bt.value_ = rset.getDouble("val");
            weight = 1;
          }
        }
        bt.value_ /= weight;
        basis_skin_temp_.add(bt);
      }
    } else {
      while (rset.next()){
        TSEBasisTemp bt = new TSEBasisTemp();
        bt.setTS(rset.getString("tbegin"));
        bt.value_ = rset.getDouble("val");
        basis_skin_temp_.add(bt);
      }
    }
    rset.close();
    stmt.close();
  }

  private void loadBasisAirTemp(Connection conn) throws SQLException
  {
    basis_air_temp_ = new Vector<TSEBasisTemp>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, val from basis_air_temp where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    if (Filtered){
      int weight = 1;
      TSEBasisTemp bt = new TSEBasisTemp();
      TSEvent tmp = new TSEvent();
      Date time_next, time_two_next;
      int filter_inc = 1000*60*5;
      if (rset.next()){
        bt.setTS(rset.getString("tbegin"));
        bt.value_ = rset.getDouble("val");
        time_next = new Date(bt.ts_.getTime() + filter_inc); // Time + 5 minutes.
        time_two_next = new Date(time_next.getTime() + filter_inc); // Time + 10 minutes;

        while (rset.next()){
          tmp.setTS(rset.getString("tbegin"));
          if (tmp.ts_.before(time_next)){
            bt.value_ += rset.getDouble("val");
            weight++;
          } else {
            bt.value_ /= weight;
            basis_air_temp_.add(bt);
            if (tmp.ts_.before(time_two_next)){
              bt = new TSEBasisTemp(time_next);
              time_next = time_two_next;
            } else {
              bt = new TSEBasisTemp(time_next);  // add a null value to show that there was a gap in data
              bt.value_ = 0;
              basis_air_temp_.add(bt);
              bt = new TSEBasisTemp(tmp.ts_);
              time_next = new Date(bt.ts_.getTime() + filter_inc);
            }
            time_two_next = new Date(time_next.getTime() + filter_inc);
            bt.value_ = rset.getDouble("val");
            weight = 1;
          }
        }
        bt.value_ /= weight;
        basis_air_temp_.add(bt);
      }
    } else {
      while (rset.next()){
        TSEBasisTemp bt = new TSEBasisTemp();
        bt.setTS(rset.getString("tbegin"));
        bt.value_ = rset.getDouble("val");
        basis_air_temp_.add(bt);
      }
    }
    rset.close();
    stmt.close();
  }

  private void loadBasisActivity(Connection conn) throws SQLException
  {
    basis_activity_ = new Vector<TSEBasisActivity>();
    String query = "Select to_char(ts, 'dd-mm-yyyy hh24:mi:ss') as tbegin, type from basis_activity where pid = "
        + id_ + " Order by ts";
    Statement stmt = conn.createStatement();
    ResultSet rset = stmt.executeQuery(query);
    while (rset.next()){
      TSEBasisActivity ba = new TSEBasisActivity();
      ba.setTS(rset.getString("tbegin"));
      ba.value_ = rset.getString("type");
      basis_activity_.add(ba);
    }
    rset.close();
    stmt.close();
  }


  public void loadFromXML()
  {
    
  }
  

  public void saveIntoXML(String xml_file) throws IOException
  {
    Serializer ser = new Serializer(new FileOutputStream(xml_file, false), "UTF-8");
    ser.setIndent(2);
    ser.setMaxLength(0);

    Element e_doc = new Element("patient");
    e_doc.addAttribute(new Attribute("id", String.valueOf(id_)));
    e_doc.addAttribute(new Attribute("weight", String.valueOf(weight_)));
    e_doc.addAttribute(new Attribute("insulin_type", insulin_type_));
    e_doc.appendChild(getInsulinSensitivityElement());
    e_doc.appendChild(getCarbRatioElement());
    e_doc.appendChild(getGlucoseLevelElement());
    e_doc.appendChild(getFingerStickElement());
    e_doc.appendChild(getBasalElement());
    e_doc.appendChild(getTempBasalElement());
    e_doc.appendChild(getBolusElement());
    e_doc.appendChild(getMealElement());
    e_doc.appendChild(getSleepElement());
    e_doc.appendChild(getWorkElement());
    e_doc.appendChild(getInfusionSetElement());
    e_doc.appendChild(getPumpFailureElement());

    if (All){
      e_doc.appendChild(getStressorsElement());
      e_doc.appendChild(getMensesElement());
      e_doc.appendChild(getHypoEventElement());
      e_doc.appendChild(getHypoActionElement());
      e_doc.appendChild(getMiscEventElement());
      e_doc.appendChild(getIllnessElement());
      e_doc.appendChild(getExerciseElement());
    }
    if (Basis){
      e_doc.appendChild(getBasisHeartRateElement());
      e_doc.appendChild(getBasisGSRElement());
      e_doc.appendChild(getBasisSkinTempElement());
      e_doc.appendChild(getBasisAirTempElement());

      e_doc.appendChild(getBasisStepsElement());
      e_doc.appendChild(getBasisCaloriesElement());
      e_doc.appendChild(getBasisActivityElement());
      e_doc.appendChild(getBasisActionElement());
      e_doc.appendChild(getBasisSleepElement());
    }

    Document doc = new Document(e_doc);
    ser.write(doc);
    ser.flush();
  }
  
  
  private Element getInsulinSensitivityElement()
  {
    Element e_all = new Element("insulin_sensitivity");
    for (TSEInsulinSensitivity tse: insulin_sensitivity_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getCarbRatioElement()
  {
    Element e_all = new Element("carb_ratio");
    for (TSECarbRatio tse: carb_ratio_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getGlucoseLevelElement()
  {
    Element e_all = new Element("glucose_level");
    for (TSEGlucoseLevel tse: glucose_level_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getFingerStickElement()
  {
    Element e_all = new Element("finger_stick");
    for (TSEFingerStick tse: finger_stick_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getBasalElement()
  {
    Element e_all = new Element("basal");
    for (TSEBasal tse: basal_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getTempBasalElement()
  {
    Element e_all = new Element("temp_basal");
    for (TSETempBasal tse: temp_basal_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts_begin", tse.getTS()));
      e_tse.addAttribute(new Attribute("ts_end", tse.getEndTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getBolusElement()
  {
    Element e_all = new Element("bolus");
    for (TSEBolus tse: bolus_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts_begin", tse.getTS()));
      e_tse.addAttribute(new Attribute("ts_end", tse.getEndTS()));
      e_tse.addAttribute(new Attribute("type", tse.getType()));
      e_tse.addAttribute(new Attribute("dose", String.valueOf(tse.dose_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getMealElement()
  {
    Element e_all = new Element("meal");
    for (TSEMeal tse: meal_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("type", tse.type_));
      e_tse.addAttribute(new Attribute("carbs", String.valueOf(tse.carbs_)));
      if (All){
        for (TSEFood tse_f: tse.get_food()){
          Element e_tse_f = new Element("food");
          e_tse_f.addAttribute(new Attribute("name", tse_f.food_name_));
          e_tse_f.addAttribute(new Attribute("amount", String.valueOf(tse_f.amount_)));
          e_tse_f.addAttribute(new Attribute("units", tse_f.units_));
          e_tse_f.addAttribute(new Attribute("alcoholic", tse_f.alcoholic_));
	  e_tse.appendChild(e_tse_f);
        }
      }
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getSleepElement()
  {
    Element e_all = new Element("sleep");
    for (TSESleep tse: sleep_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts_begin", tse.getTS()));
      e_tse.addAttribute(new Attribute("ts_end", tse.getEndTS()));
      if(All){
        e_tse.addAttribute(new Attribute("quality", String.valueOf(tse.quality_)));
      }
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getWorkElement()
  {
    Element e_all = new Element("work");
    for (TSEWork tse: work_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts_begin", tse.getTS()));
      e_tse.addAttribute(new Attribute("ts_end", tse.getEndTS()));
      e_tse.addAttribute(new Attribute("intensity", String.valueOf(tse.intensity_)));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getInfusionSetElement()
  {
    Element e_all = new Element("infusion_set");
    for (TSEInfusionSet tse: infusion_set_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getPumpFailureElement()
  {
    Element e_all = new Element("pump_failure");
    for (TSEPumpFailure tse: pump_failure_) {
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts_begin", tse.getTS()));
      e_tse.addAttribute(new Attribute("ts_end", tse.getEndTS()));
      e_tse.addAttribute(new Attribute("diagnostic", tse.diagnostic_));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getStressorsElement()
  {
    Element e_all = new Element("stressors");
    for (TSEStressors tse: stressors_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("type", tse.type_));
      e_tse.addAttribute(new Attribute("description", tse.description_));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getMensesElement()
  {
    Element e_all = new Element("menses");
    for (TSEMenses tse: menses_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("description", tse.description_));
      e_tse.addAttribute(new Attribute("symptom_severity", String.valueOf(tse.severity_)));
      e_all.appendChild(e_tse);
    }

    return e_all;
  }

  
  private Element getHypoEventElement()
  {
    Element e_all = new Element("hypo_event");
    for (TSEHypoEvent tse: hypo_event_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      for (String symptom: tse.getSymptoms()){
        Element e_symptom = new Element("symptom");
        e_symptom.addAttribute(new Attribute("name", symptom));
        e_tse.appendChild(e_symptom);
      }
      e_all.appendChild(e_tse);
    }

    return e_all;
  }


  private Element getHypoActionElement()
  {
    Element e_all = new Element("hypo_action");
    for (TSEHypoAction tse: hypo_action_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("action", tse.action_));
      e_tse.addAttribute(new Attribute("carbs", String.valueOf(tse.carbs_)));
      e_all.appendChild(e_tse);
    }

    return e_all;
  }


  private Element getMiscEventElement()
  {
    Element e_all = new Element("misc_event");
    for (TSEMiscEvent tse: misc_event_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("description", tse.description_));
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getIllnessElement()
  {
    Element e_all = new Element("illness");
    for (TSEIllness tse: illness_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts_begin", tse.getTS()));
      e_tse.addAttribute(new Attribute("ts_end", tse.getEndTS()));
      e_tse.addAttribute(new Attribute("type", tse.type_));
      e_tse.addAttribute(new Attribute("description", tse.description_));
      for (String symptom: tse.getSymptoms()){
        Element e_sym = new Element("symptom");
        e_sym.addAttribute(new Attribute("name", symptom));
        e_tse.appendChild(e_sym);
      }
      e_all.appendChild(e_tse);
    }
    
    return e_all;
  }


  private Element getExerciseElement(){
    Element e_all = new Element("exercise");
    for (TSEExercise tse: exercise_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("intensity", String.valueOf(tse.intensity_)));
      e_tse.addAttribute(new Attribute("type", tse.type_));
      e_tse.addAttribute(new Attribute("duration", String.valueOf(tse.duration_)));
      e_tse.addAttribute(new Attribute("competitive", tse.competitive_));
      e_all.appendChild(e_tse);
    }

    return e_all;
  }


  private Element getBasisHeartRateElement(){
    Element e_all = new Element("basis_heart_rate");
    for (TSEBasisHeartRate tse: basis_heart_rate_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisGSRElement(){
    Element e_all = new Element("basis_gsr");
    for (TSEBasisGSR tse: basis_gsr_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisStepsElement(){
    Element e_all = new Element("basis_steps");
    for (TSEBasisSteps tse: basis_steps_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisCaloriesElement(){
    Element e_all = new Element("basis_calories");
    for (TSEBasisCalories tse: basis_calories_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisSkinTempElement(){
    Element e_all = new Element("basis_skin_temperature");
    for (TSEBasisTemp tse: basis_skin_temp_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisAirTempElement(){
    Element e_all = new Element("basis_air_temperature");
    for (TSEBasisTemp tse: basis_air_temp_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", String.valueOf(tse.value_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisActivityElement(){
    Element e_all = new Element("basis_activity");
    for (TSEBasisActivity tse : basis_activity_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("value", tse.value_));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  private Element getBasisActionElement(){
    Element e_all = new Element("basis_action");
    for (TSEBasisAction tse: basis_action_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("ts", tse.getTS()));
      e_tse.addAttribute(new Attribute("type", tse.type_));
      e_tse.addAttribute(new Attribute("seconds", String.valueOf(tse.seconds_)));
      e_tse.addAttribute(new Attribute("steps", String.valueOf(tse.steps_)));
      e_tse.addAttribute(new Attribute("calories", String.valueOf(tse.cal_)));
      e_all.appendChild(e_tse);
    }
    return e_all;
  }

  private Element getBasisSleepElement(){
    Element e_all = new Element("basis_sleep");
    for (TSEBasisSleep tse: basis_sleep_){
      Element e_tse = new Element("event");
      e_tse.addAttribute(new Attribute("tbegin", tse.getTS()));
      e_tse.addAttribute(new Attribute("tend", tse.getEndTS()));
      e_tse.addAttribute(new Attribute("quality", String.valueOf(tse.quality_)));
      if (tse.type_ == null){
       System.out.println("Type == Null");
       e_tse.addAttribute(new Attribute("type", ""));
      }
      else {
       e_tse.addAttribute(new Attribute("type", tse.type_));
      }
      for (TSEvent tse2: tse.getTossAndTurn()){
        Element e_tse2 = new Element("toss_and_turn");
        e_tse2.addAttribute(new Attribute("ts", tse2.getTS()));
        e_tse.appendChild(e_tse2);
      }
      e_all.appendChild(e_tse);
    }
    return e_all;
  }


  public Vector<TSEGlucoseLevel> getGlucoseLevels()
  {
    return glucose_level_;
  }
  
  public Vector<TSEBasal> getBasalValues()
  {
    return basal_;
  }
  
  public Vector<TSETempBasal> getTempBasalValues()
  {
    return temp_basal_;
  }
  
  public Vector<TSEBolus> getBoluses()
  {
    return bolus_;
  }
  
  public Vector<TSEMeal> getMeals()
  {
    return meal_;
  }
  
  public Vector<TSEInsulinSensitivity> getInsulinSensitivity()
  {
    return insulin_sensitivity_;
  }
  
  public Vector<TSECarbRatio> getCarbRatio()
  {
    return carb_ratio_;
  }

  public Vector<TSEStressors> getStressors()
  {
    return stressors_;
  }

  public Vector<TSEMenses> getMenses()
  {
    return menses_;
  }

  public Vector<TSEHypoEvent> getHypoEvent()
  {
    return hypo_event_;
  }

  public Vector<TSEHypoAction> getHypoAction()
  {
    return hypo_action_;
  }

  public Vector<TSEMiscEvent> getMiscEvent()
  {
    return misc_event_;
  }

  public Vector<TSEIllness> getIllness()
  {
    return illness_;
  }

  public Vector<TSEExercise> getExercise()
  {
    return exercise_;
  }

  public Vector<TSEBasisSleep> getBasisSleep()
  {
    return basis_sleep_;
  }

  public Vector<TSEBasisAction> getBasisAction()
  {
    return basis_action_;
  }

  public Vector<TSEBasisHeartRate> getBasisHeartRate()
  {
    return basis_heart_rate_;
  }

  public Vector<TSEBasisSteps> getBasisSteps()
  {
    return basis_steps_;
  }

  public Vector<TSEBasisCalories> getBasisCalories()
  {
    return basis_calories_;
  }

  public Vector<TSEBasisGSR> getBasisGsr()
  {
    return basis_gsr_;
  }

  public Vector<TSEBasisTemp> getBasisSkinTemp()
  {
    return basis_skin_temp_;
  }

  public Vector<TSEBasisTemp> getBasisAirTemp()
  {
    return basis_air_temp_;
  }

  public Vector<TSEBasisActivity> getBasisActivity()
  {
    return basis_activity_;
  }

  public Vector<TSEFingerStick> getFingerSticks()
  {
    return finger_stick_;
  }

  public Vector<TSESleep> getSleep()
  {
    return sleep_;
  }

  
  public static void main(String args[])
  {
    if (args.length < 2 && args.length > 5) {
      System.err.println("Usage: java OraclePatient [--all] [--basis[ --filtered]] <pid> <xml-file>!");
      System.err.println("  --all       - Include all data in output");
      System.err.println("  --basis     - Included basis data in output");
      System.err.println("  --filtered  - Only used when --basis flag is included. Filters out basis data");
      System.err.println("                that is not used in plotting program. Avg basis data every 5 min.");
      System.exit(1);
    }
    int i = 0;
    if (args.length == 3){
      if (args[i].equals("--all")){
        All = true;
      } else if (args[i].equals("--basis")){
        Basis = true;
      } else {
        System.err.println("Usage: java OraclePatient [--all] [--basis[ --filtered]] <pid> <xml-file>!");
        System.exit(1);
      }
      ++i;
    } else if (args.length == 4){
      if (args[i].equals("--all") && args[i+1].equals("--basis")){
        All = true;
        Basis = true;
      } else if (args[i].equals("--basis") && args[i+1].equals("--filtered")){
        Basis = true;
        Filtered = true;
      } else {
        System.err.println("Usage: java OraclePatient [--all] [--basis[ --filtered]] <pid> <xml-file>!");
        System.exit(1);
      }
      i = i + 2;
    } else if (args.length == 5){
     if (args[i].equals("--all") && args[i+1].equals("--basis") && args[i+2].equals("--filtered")){
       All = true;
       Basis = true;
       Filtered = true;
     } else {
        System.err.println("Usage: java OraclePatient [--all] [--basis[ --filtered]] <pid> <xml-file>!");
        System.exit(1);
     }
     i = i + 3;
    }

    OraclePatient op = new OraclePatient(Integer.parseInt(args[i++]));
    
    try {
      // Read patient data from Oracle DB.
      Connection conn = op.getConnection();
      op.loadFromDB(conn);

      // Output patient data into new XML file.
      op.saveIntoXML(args[i]);

	ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream("op"+args[i-1]+".ser"));
      out.writeObject(op);
      out.close();
      
      conn.close();
    }
    catch (SQLException e) {
      e.printStackTrace();
    }
    catch (IOException e) {
      e.printStackTrace();
    }
  }
}
