package dbpatient;

import java.text.ParseException;
import java.util.Date;
import java.util.HashMap;

public class TSEBolus extends TSEvent
{
  private static final long serialVersionUID = 1L;

  static public enum BolusType
  {
    NORMAL("normal"), NORMAL_DUAL("normal dual"), SQUARE("square"), SQUARE_DUAL("square dual");
    
    private final String name_;
    BolusType(String name)
    {
      name_ = name;
    }
  }
  
  static protected HashMap<String, BolusType> btypes_;
  static {
    btypes_ = new HashMap<String, BolusType>();
    for (BolusType bt: BolusType.values())
      btypes_.put(bt.name_, bt);
  }
  
  // Bolus info read from the Oracle DB.
  public BolusType btype_;
  public double dose_;
  
  // Use inherited ts_ as begin_bolus_.
  public Date end_bolus_;
  
  public void setType(String type)
  {
    // Add Support for 400 patients while retaining support for 300 patients and keep same output
    if (type.toLowerCase().equals("dual (normal part)")){
     type = "normal dual";
    }
    else if (type.toLowerCase().equals("dual (square part)")){
     type = "square dual";
    }
    btype_ = btypes_.get(type.toLowerCase());
    if (btype_ == null) {
      System.err.println("unrecognized bolus type: " + type + "!");
      System.exit(1);
    }
  }
  
  
  public String getType()
  {
    return btype_.name_;
  }
  
  
  public void setEndTS(String tstamp)
  {
    try {
      end_bolus_ = sdf_.parse(tstamp);
    }
    catch (ParseException e) {
      e.printStackTrace();
      System.exit(1);
    }
    catch (java.lang.NullPointerException e){
      end_bolus_ = null;
    }
  }
  

  public String getEndTS()
  {
    if (end_bolus_ == null){
      return "";
    }
    else {
      return sdf_.format(end_bolus_);
    }
  }
}
