package kalman;

import Jama.Matrix;

public abstract class ExtendedKalmanFilter
{
  // Current time.
  protected int t_;
  // Mean X of system state vector.
  protected Matrix X_;
  // Covariance P of system state vector.
  protected Matrix P_;
  
  abstract public Matrix computeNextState();
  abstract public Matrix getStateCovarianceQ();
  abstract public Matrix getStateJacobianF();

  abstract public Matrix computeObservation();
  abstract public Matrix getObsCovarianceR();
  abstract public Matrix getObsJacobianH();
  
  
  public void predictState()
  {
    Matrix Q = getStateCovarianceQ();
    Matrix F = getStateJacobianF();

    // Update mean X of state vector.
    X_ = computeNextState();
        
    // Update covariance P of state vector.
    P_ = F.times(P_).times(F.transpose()).plus(Q);
    
    // Update time.
    t_++;
  }
  
  
  public void correctState(Matrix O)
  {
    Matrix R = getObsCovarianceR();
    Matrix H = getObsJacobianH();
    
    // Compute Kalman gain K.
    Matrix Kf = H.times(P_).times(H.transpose()).plus(R);
    Matrix K = P_.times(H.transpose()).times(Kf.inverse());
    
    // Update mean X of state vector.
    X_ = X_.plus(K.times(O.minus(computeObservation())));
    
    // Update covariance P of state vector.
    P_ = P_.minus(K.times(H.times(P_)));
  }
  
  
  public void correctStateDuke(Matrix O)
  {
    Matrix R = getObsCovarianceR();
    Matrix H = getObsJacobianH();
    
    // Compute Kalman gain K.
    Matrix Kf = H.times(P_).times(H.transpose()).plus(R);
    Matrix K = P_.times(H.transpose()).times(Kf.inverse());
    
    // Update mean X of state vector.
    X_ = X_.plus(K.times(O.minus(computeObservation())));
    
    // Update covariance P of state vector.
    P_ = P_.minus(K.times(H.times(P_.times(H.transpose())).plus(R)).times(K.transpose()));
  }
  
}
