package physiology;

import java.io.Serializable;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Vector;

import dbpatient.TSEGlucoseLevel;

public class BGLValues implements Serializable
{
  private static final long serialVersionUID = 1L;
  
  public static final int DELTA_TIME_ = 1;

  // BGL measurements, in chronological order.
  Vector<TSEGlucoseLevel> bgl_;
  
  // Current timestamp value.
  Date ts_;
  
  // Position of next BGL measurement.
  int pos_;
  
  transient static Calendar calendar_ = new GregorianCalendar();
  
  
  public BGLValues(Vector<TSEGlucoseLevel> bgl)
  {
    bgl_ = bgl;
    
    rewind();
  }
  
  
  // Reset iterator to beginning of sequence.
  public void rewind()
  {
    ts_ = addMinutes(bgl_.get(0).ts_, -DELTA_TIME_); 
    pos_ = 0;    
  }
  
  
  protected Date addMinutes(Date date, int minutes)
  {
    calendar_.setTime(date);
    calendar_.add(Calendar.MINUTE, minutes);
    
    return calendar_.getTime();
  }
  
  
  public DeltaData nextDeltaData()
  {
    // No BGL measurements available.
    if (pos_ >= bgl_.size())
      return null;
    
    TSEGlucoseLevel bgl = bgl_.get(pos_);

    // Initialize DeltaData structure.
    DeltaData dd = new DeltaData();
    dd.begin_ = ts_;

    Date next_ts = addMinutes(ts_, DELTA_TIME_);
    Date next_measure_ts = bgl.ts_;
    
    // Next BGL measurement is within DELTA_TIME_ minutes of last measurement.
    if (next_measure_ts.compareTo(next_ts) <= 0) {
      dd.end_ = next_measure_ts;
      dd.bgl_ = (double) bgl.value_;
      ts_ = next_measure_ts;
      pos_++;
    }
    else { // No BGL measurement during the next 5 minutes.
      dd.end_ = next_ts;
      dd.bgl_ = null;
      ts_ = next_ts;
    }
    
    return dd;
  }
}
