package physiology;

import java.io.Serializable;
import java.util.Date;
import java.util.Vector;

import dbpatient.TSEBasal;
import dbpatient.TSETempBasal;

public class BasalValues implements Serializable
{
  private static final long serialVersionUID = 1L;

  // Daily patterns.
  DailyPatternValues dpv_;
  
  // Temporary basals.
  Vector<TSETempBasal> temp_basal_;
  int pos_temp_;
  
  
  public BasalValues(Vector<TSEBasal> basals, Vector<TSETempBasal> temp_basals)
  {
    // Set daily patterns.
    dpv_ = new DailyPatternValues(basals);
    
    // Set temporary basal rates.
    temp_basal_ = temp_basals;
    
    rewind();
  }
  
  
  public void rewind()
  {
    dpv_.rewind();
    pos_temp_ = 0;
  }
  
  /**
   * This should return the insulin amount delivered between (begin, end] from
   * the basal rate. Since the interval is small (DELTA_TIME_ min), assume for simplicity
   * that the rate is constant, and use the rate corresponding to end_.
   * 
   * In the future, change to account for cases where basal rate changes during 
   * the interval (begin, end].
   * 
   * @param begin Beginning of time interval.
   * @param end End of time interval.
   * @return The amount of basal insulin delivered during the interval. 
   */
  public Double getBasal(Date begin, Date end)
  {
    // Find temporary basal, if any.
    double temp_basal = Double.NaN;
    while (pos_temp_ < temp_basal_.size()) {
      TSETempBasal tse = temp_basal_.get(pos_temp_);
      if (end.after(tse.end_temp_basal_))
        pos_temp_++;
      else {
        if (end.compareTo(tse.ts_) >= 0)
          temp_basal = tse.value_;
        break;
      }
    }
    
    if (!Double.isNaN(temp_basal))
      return temp_basal;
    
    // Find daily pattern event, if any.
    TSEBasal basal = (TSEBasal) dpv_.getPatternEvent(begin, end);
    if (basal == null)
      return null;
    
    return basal.value_;
  }
}
