package physiology;

import java.io.Serializable;
import java.util.Date;
import java.util.Vector;

import dbpatient.TSEBolus;

public class BolusValues implements Serializable
{
  private static final long serialVersionUID = 1L;

  // Bolus dosages, in chronological order.
  Vector<TSEBolus> bolus_;
  // Current position of bolus.
  int pos_;
  
  
  public BolusValues(Vector<TSEBolus> bolus)
  {
    bolus_ = bolus;
    
    rewind();
  }
  
  
  public void rewind()
  {
    pos_ = 0;
  }
  
  /**
   * Assume getBolus() is always called with intervals in chronological order.
   * @param begin
   * @param end
   * @return Amount of insulin delivered in [begin, end) through bolus.
   */
  public Double getBolus(Date begin, Date end)
  {
    // Find closest bolus event, if any.
    while (pos_ < bolus_.size()) {
      TSEBolus tse = bolus_.get(pos_);
      if (tse.end_bolus_.compareTo(begin) >= 0)
        break;
      pos_++;
    }
    
    if (pos_ >= bolus_.size())
      return null;
    
    TSEBolus tse = bolus_.get(pos_);
    if (tse.ts_.compareTo(end) >= 0)
      return null;
    
    // Normal bolus overlapping with [begin, end).
    if (tse.btype_ == TSEBolus.BolusType.NORMAL || tse.btype_ == TSEBolus.BolusType.NORMAL_DUAL)
      return tse.dose_;
    
    // Square bolus overlapping with [begin, end).
    long bolus_duration = tse.end_bolus_.getTime() - tse.ts_.getTime();

    if (bolus_duration == 0){ return tse.dose_; } // if square bolus has 0 duration treat like normal bolus

    long overlap_duration = end.getTime() - Math.max(begin.getTime(), tse.ts_.getTime());
    
    return tse.dose_ * overlap_duration / (double) bolus_duration;
  }
  
}
