package BGPredict;
import java.io.*;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.Calendar;
import java.util.List;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;
import util.DateUtil;
import util.DBUtil;
import Diabetes.DataObjects.Reading;
import BGPredict.ComputeExampleVector;
import java.lang.Math;
import BGPredict.ExampleVector;
import BGPredict.TestPoint;

public class Experiment101 extends PredictionConfig {

    public Experiment101(){
        super(0);
        //targets = new ArrayList<Double>();
        //predictions = new ArrayList<Double>();
        //pids = new ArrayList<Integer>();
        //ts = new ArrayList<String>();
        //t0s = new ArrayList<Double>();
        points = new ArrayList<TestPoint>();
    }



    protected List<TestPoint> points;


  

    public void printResults_p()
    {
    		double RMSE = 0;
    		double t0RMSE = 0;
        try
            {
                FileWriter out_file = new FileWriter(OUTPUT);
                out_file.write("Window = " + WINDOW);
                out_file.write("\nScale = " + SCALE);
                out_file.write("\nKernel = " + KERNEL);
                out_file.write("\nPrediction = " + LOOKAHEAD + " minutes\n");
                for(int i = 0; i < points.size(); ++i)
                    {
                        RMSE += (points.get(i).Target - points.get(i).Prediction) * (points.get(i).Target - points.get(i).Prediction);
                        double error = points.get(i).Prediction - points.get(i).Target;
                        t0RMSE += (points.get(i).Target - points.get(i).t0) * (points.get(i).Target - points.get(i).t0);
                        out_file.write(points.get(i).pid + " " + points.get(i).TimeStamp + " target:" + points.get(i).Target + " t0: " + points.get(i).t0 + " predicted:" + points.get(i).Prediction + " error:" + String.format("%.2f", error) + "\n");
                    }
                RMSE = Math.sqrt(RMSE/points.size());
                t0RMSE = Math.sqrt(t0RMSE/points.size());
                out_file.write("\n Macro RMSE = " + RMSE);
                out_file.write("\n t0 RMSE = " + t0RMSE); 
                out_file.close();
            }
        catch(IOException e)
            {
                System.out.println(e);
            }
  	}


    private double scale(double min, double max, double val){
        if(SCALE == true){
            return ((val-min)/(max-min));
        }else{
            return val;
        }
    }


    public void arima() throws IOException, FileNotFoundException{
        char[] progress = {'|','/','-','\\'};
        // int[] patients = {304,306,310,312,316};
        int[] patients = {101};
        for(int i = 0; i < patients.length; ++i){
            int pid = patients[i];
            try{
                DBUtil.read_op("op" + pid + ".ser");
            }catch(ClassNotFoundException e){
                System.out.println(e);
            }
            List<Reading> fingerstics = DBUtil.op_getFingersticks(null,null);
            Reading first_reading = DBUtil.op_nextCGMS(fingerstics.get(0).getT());
            GregorianCalendar gc = DateUtil.createGC(DateUtil.makeTS(first_reading.getT()));
            List<Reading> cgms = DBUtil.op_getCGMS(null,null);
            FileWriter out = new FileWriter(pid +"arima_in.dat");
            out.write("date time");
            for(int j = 0; j < 4*24*60/5; ++j){
                out.write(",t" + j);
            }
            out.write("\n");
            System.out.println("on patient " + pid);
            System.out.print("|");
            for(int j = 0; j < cgms.size(); ++j){
                if(cgms.get(j).getT().before(first_reading.getT())){
                    continue;
                }
                System.out.print("\b" + progress[j%progress.length]);
                GregorianCalendar gc2 = DateUtil.createGC(DateUtil.makeTS(cgms.get(j).getT()));
                gc2.add(Calendar.HOUR,-4*24);
                List<Reading> arima_values = DBUtil.op_getCGMS(gc2,cgms.get(j).getT());
                while (arima_values.size() > 1151) {
                    arima_values.remove(0);
                }
                out.write(DateUtil.makeTS(cgms.get(j).getT()));
                for(int k = 0; k < arima_values.size(); ++k){
                    out.write("," + arima_values.get(k).getValue());
                }
                out.write("\n");
            }
            System.out.print("\b");
            out.close();
        }
    }

  

    public void run_tuning_p() throws IOException, FileNotFoundException
    {

        /*
        ** This code creates the files in datafiles necessary for tuning. 
        ** Performing this separately from the normal running of an experiment saves computation time 
        ** and hard drive space. This step only needs to be done before tuning, not during every run
        ** of the experiment.
        */
	
        System.out.println("Window = " + WINDOW + " weeks");
        System.out.println("Scale = " + SCALE);
      	System.out.println("Kernel = " + KERNEL);
      	System.out.println("Prediction = " + LOOKAHEAD + " minutes");
      	char[] progress = {'|','/','-','\\'};

      	BufferedReader in = new BufferedReader(new FileReader(DATASET));
        List<Double> MinValues = null;
      	List<Double> MaxValues = null;
        int point_number = 0;

      
      	String Line;

        //read in test points
        while((Line = in.readLine()) != null)
            {
                if(Line.length() == 0 || Line.charAt(0) == '#')
                    {
                        continue;
                    }
                String LineArr[] = Line.split("\t");
                if(LineArr.length < 3)
                    {
                        break;
                    }

                point_number++;
                TestPoint point = new TestPoint(Integer.parseInt(LineArr[0]), LineArr[1] + " " + LineArr[2], point_number);
                points.add(point);
       	
            }
        //done reading in test points
        in.close();

	
        int cur_point = 0;
        for(int i = 0; i < (int)(points.size()); ++i)
            {
                TestPoint point = points.get(i);

                //open input files
                BufferedReader physiological_model = new BufferedReader(new FileReader(point.pid+".predicts"));
                BufferedReader physiological_model_test = new BufferedReader(new FileReader(point.pid+"test.pred"));
                BufferedReader arima_models = new BufferedReader(new FileReader(point.pid+"arima_out.dat"));
                BufferedReader arima_test = new BufferedReader(new FileReader(point.pid+"arima_out.dat"));
                ExampleVector exampleVector = null;
                try
                    {
			
                        DBUtil.read_op("op" + point.pid + ".ser");
		
                    }
                catch(ClassNotFoundException e)
                    {
                        System.out.println(e);
                    }


                List<ExampleVector > trainVectors = new ArrayList<ExampleVector >();

                List<Reading> fingerstics = DBUtil.op_getFingersticks(null,null);
                Reading first_reading = DBUtil.op_nextCGMS(fingerstics.get(0).getT());
                GregorianCalendar gc2 = DateUtil.createGC(DateUtil.makeTS(first_reading.getT()));

                GregorianCalendar gclast = DateUtil.createGC(DateUtil.makeTS(point.gc));
                gclast.add(Calendar.MINUTE, -1 * LOOKAHEAD);
                //System.out.println("PID " + point.pid + ", " + DateUtil.makeTS(point.gc));
                GregorianCalendar gcmax = DateUtil.createGC(DateUtil.makeTS(gclast));
                gcmax.add(Calendar.HOUR, -24*7*WINDOW);

		
		
                //System.out.print(progress[trainVectors.size()%progress.length]);
                while(gc2.before(gclast))
                    {
                        Reading valid = DBUtil.op_nextCGMS(gc2);
                        if(point.gc.before(valid.getT()))
                            {
                                continue;
                            }
        
                        gc2 = DateUtil.createGC(DateUtil.makeTS(valid.getT()));
                        ComputeExampleVector CEV = new ComputeExampleVector(point.pid,gc2,physiological_model,arima_models);
                        CEV.computeVector();
                        exampleVector = CEV.getVector();
                        trainVectors.add(exampleVector);
                        //System.out.print("\b" + progress[trainVectors.size()%progress.length]);

                        if(MinValues == null)
                            {
                                MinValues = new ArrayList<Double>();
                                MaxValues = new ArrayList<Double>();
                                for(int j = 0; j < exampleVector.vector.size(); ++j)
                                    {
                                        MinValues.add(exampleVector.vector.get(j));
                                        MaxValues.add(exampleVector.vector.get(j)+.01);
                                    }
                                continue;
                            }
                        if(exampleVector.vector.size() != MinValues.size())
                            {
                                System.out.println("error: " + DateUtil.makeTS(gc2) + " example vector not the right size = ");
                                for(int j = 0; j < exampleVector.vector.size(); ++j)
                                    {
                                        System.out.print(j + ": " + exampleVector.vector.get(j) + " ");
                                    }
                                System.out.println("");
                            }
                        for(int j = 0; j < exampleVector.vector.size() && exampleVector.vector.size() == MinValues.size(); ++j)
                            {
                                if(exampleVector.vector.get(j) < MinValues.get(j))
                                    {
                                        MinValues.set(j,exampleVector.vector.get(j));
                                    }
                                if(exampleVector.vector.get(j) > MaxValues.get(j))
                                    {
                                        MaxValues.set(j,exampleVector.vector.get(j));
                                    }
                            }
                    }
                int count_vectors = 0;
                int count_dev_vectors = 0;
                GregorianCalendar windowGC = DateUtil.createGC(DateUtil.makeTS(point.gc));
                windowGC.add(Calendar.HOUR,-168*WINDOW);

                String train_file_name = "datafiles/" + point.Point_number + "train.dat";
                String test_file_name = "datafiles/" + point.Point_number + "test.dat";
                String tuning_file_name = "datafiles/" + point.Point_number + "dev.dat";

                FileWriter out2 = new FileWriter(train_file_name);
                for(int j = 0; j < (int)(trainVectors.size()); ++j)
                    {
                        if(trainVectors.get(j).gc.before(windowGC))
                            {
                                continue;
                            }

                        ++count_vectors;
			
                        //get target
                        out2.write(trainVectors.get(j).vector.get(0) + " ");
                        //skip the scaling for t0

                        // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60
                        for(int k = 1; k < trainVectors.get(j).vector.size(); ++k)
                            {
                                out2.write(k + ":" + scale(MinValues.get(k), MaxValues.get(k), trainVectors.get(j).vector.get(k)) + " ");
                            }
                        out2.write("\n");
                    }

		

                out2.close();

                out2 = new FileWriter(tuning_file_name);
                for(int k = 0; k < (int)(trainVectors.size()); ++k)
                    {
                        if(!trainVectors.get(k).gc.before(windowGC))
                            {
                                if(count_dev_vectors > 0)
                                    {
                                        break;
                                    }
                            }
                        ++count_dev_vectors;
		      
                        //write target
                        out2.write(trainVectors.get(k).vector.get(0) + " ");
                        // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60
                        for(int j = 1; j < trainVectors.get(k).vector.size(); ++j)
                            {
                                out2.write(j + ":" + scale(MinValues.get(j), MaxValues.get(j), trainVectors.get(k).vector.get(j)) + " ");
                            }
                        out2.write("\n");
                    }
                out2.close();
		
                //System.out.println("\n" + count_vectors + " training vectors " + count_dev_vectors + " dev vectors");
                out2 = new FileWriter(test_file_name);
                ComputeExampleVector CEV = new ComputeExampleVector(point.pid,point.gc,physiological_model_test,arima_test);
                CEV.computeVector();
                exampleVector = CEV.getVector();
		
                //get target points

                out2.write(exampleVector.vector.get(0) + " ");

                points.get(i).Target = exampleVector.vector.get(0);

                // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60

                points.get(i).t0 = exampleVector.vector.get(1);

                // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60      
                for(int j = 1; j < exampleVector.vector.size(); ++j)
                    {
                        out2.write(j + ":" + scale(MinValues.get(j), MaxValues.get(j), exampleVector.vector.get(j)) + " ");
                    }

                out2.write("\n");
                out2.close();


                physiological_model.close();
                physiological_model_test.close();
                arima_models.close();
                arima_test.close();

                cur_point++;
                double percent = (((double)cur_point)/point_number) * 100;
                System.out.println(String.format("%.2f", percent) + "% complete");
		
            }
    }

    public void run_p() throws IOException, FileNotFoundException
    {
        System.out.println("Window = " + WINDOW + " weeks");
        System.out.println("Scale = " + SCALE);
      	System.out.println("Kernel = " + KERNEL);
      	System.out.println("Prediction = " + LOOKAHEAD + " minutes");
      	char[] progress = {'|','/','-','\\'};

      	BufferedReader in = new BufferedReader(new FileReader(DATASET));
        List<Double> MinValues = null;
      	List<Double> MaxValues = null;
        int point_number = 0;
      
      	String Line;

        //read in test points
        while((Line = in.readLine()) != null)
            {
                if(Line.length() == 0 || Line.charAt(0) == '#')
                    {
                        continue;
                    }
                // String LineArr[] = Line.split("\t");
                String LineArr[] = Line.split("[ \t]");
                if(LineArr.length < 3)
                    {
                        break;
                    }
                point_number++;
                TestPoint point = new TestPoint(Integer.parseInt(LineArr[0]), LineArr[1] + " " + LineArr[2], point_number);
                points.add(point);
       	
            }
        //done reading in test points
        in.close();

	
        int cur_point = 0;
         
        for(int i = 0; i < (int)(points.size()); ++i)
            {
                TestPoint point = points.get(i);

                //open input files
                BufferedReader physiological_model = new BufferedReader(new FileReader(point.pid+".predicts"));
                BufferedReader physiological_model_test = new BufferedReader(new FileReader(point.pid+"test.pred"));
                BufferedReader arima_models = new BufferedReader(new FileReader(point.pid+"arima_out.dat"));
                BufferedReader arima_test = new BufferedReader(new FileReader(point.pid+"arima_out.dat"));
                ExampleVector exampleVector = null;
                try
                    {
                        DBUtil.read_op("op" + point.pid + ".ser");
                    }
                catch(ClassNotFoundException e)
                    {
                        System.out.println(e);
                    }


		
                List<ExampleVector > trainVectors = new ArrayList<ExampleVector >();

                List<Reading> fingerstics = DBUtil.op_getFingersticks(null,null);
                Reading first_reading = DBUtil.op_nextCGMS(fingerstics.get(0).getT());
                GregorianCalendar gc2 = DateUtil.createGC(DateUtil.makeTS(first_reading.getT()));

                GregorianCalendar gclast = DateUtil.createGC(DateUtil.makeTS(point.gc));
                gclast.add(Calendar.MINUTE, -1 * LOOKAHEAD);
                //System.out.println("PID " + point.pid + ", " + DateUtil.makeTS(point.gc));
                GregorianCalendar gcmax = DateUtil.createGC(DateUtil.makeTS(gclast));
                gcmax.add(Calendar.HOUR, -24*7*WINDOW);

		
                //System.out.print(progress[trainVectors.size()%progress.length]);
                while(gc2.before(gclast))
                    {
                        Reading valid = DBUtil.op_nextCGMS(gc2);
                        if(point.gc.before(valid.getT()))
                            {
                                continue;
                            }
        
                        gc2 = DateUtil.createGC(DateUtil.makeTS(valid.getT()));
                        ComputeExampleVector CEV = new ComputeExampleVector(point.pid,gc2,physiological_model,arima_models);
                        CEV.computeVector();
                        exampleVector = CEV.getVector();
                        trainVectors.add(exampleVector);
                        //System.out.print("\b" + progress[trainVectors.size()%progress.length]);

                        if(MinValues == null)
                            {
                                MinValues = new ArrayList<Double>();
                                MaxValues = new ArrayList<Double>();
                                for(int j = 0; j < exampleVector.vector.size(); ++j)
                                    {
                                        MinValues.add(exampleVector.vector.get(j));
                                        MaxValues.add(exampleVector.vector.get(j)+.01);
                                    }
                                continue;
                            }
                        if(exampleVector.vector.size() != MinValues.size())
                            {
                                System.out.println("error: " + DateUtil.makeTS(gc2) + " example vector not the right size = ");
                                for(int j = 0; j < exampleVector.vector.size(); ++j)
                                    {
                                        System.out.print(j + ": " + exampleVector.vector.get(j) + " ");
                                    }
                                System.out.println("");
                            }
                        for(int j = 0; j < exampleVector.vector.size() && exampleVector.vector.size() == MinValues.size(); ++j)
                            {
                                if(exampleVector.vector.get(j) < MinValues.get(j))
                                    {
                                        MinValues.set(j,exampleVector.vector.get(j));
                                    }
                                if(exampleVector.vector.get(j) > MaxValues.get(j))
                                    {
                                        MaxValues.set(j,exampleVector.vector.get(j));
                                    }
                            }
                    }
                int count_vectors = 0;
                GregorianCalendar windowGC = DateUtil.createGC(DateUtil.makeTS(point.gc));
                windowGC.add(Calendar.HOUR,-168*WINDOW);

                String train_file_name = "datafiles/train_" + point.pid + "_" + point.TimeStamp + ".dat";
                String test_file_name = "datafiles/test_" + point.pid + "_" + point.TimeStamp + ".dat";
                String model_file_name = "datafiles/test_" + point.pid + "_" + point.TimeStamp + ".model";
                String prediction_file_name = "datafiles/test_" + point.pid + "_" + point.TimeStamp + ".out";

                FileWriter out2 = new FileWriter(train_file_name);
                for(int j = 0; j < (int)(trainVectors.size()); ++j)
                    {
                        if(trainVectors.get(j).gc.before(windowGC))
                            {
                                continue;
                            }

                        ++count_vectors;
			
                        //get target
                        out2.write(trainVectors.get(j).vector.get(0) + " ");
                        //skip the scaling for t0

                        // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60
                        for(int k = 1; k < trainVectors.get(j).vector.size(); ++k)
                            {
                                out2.write(k + ":" + scale(MinValues.get(k), MaxValues.get(k), trainVectors.get(j).vector.get(k)) + " ");
                            }
                        out2.write("\n");
                    }

		

                out2.close();
                //System.out.println("\n" + count_vectors + " training vectors ");
                out2 = new FileWriter(test_file_name);
                ComputeExampleVector CEV = new ComputeExampleVector(point.pid,point.gc,physiological_model_test,arima_test);
                CEV.computeVector();
                exampleVector = CEV.getVector();
		
                //get target points

                out2.write(exampleVector.vector.get(0) + " ");

                points.get(i).Target = exampleVector.vector.get(0);

                // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60

                points.get(i).t0 = exampleVector.vector.get(1);

                // 2 changed to 1 to get the t0 value now that lookahead is not limited to 30 and 60      
                for(int j = 1; j < exampleVector.vector.size(); ++j)
                    {
                        out2.write(j + ":" + scale(MinValues.get(j), MaxValues.get(j), exampleVector.vector.get(j)) + " ");
                    }

                out2.write("\n");
                out2.close();


                String argv[] = {"./libsvm-3.17/svm-train", "-s", "3", "-t", "2", "-c", String.valueOf(C_PARAM), "-g", String.valueOf(G_PARAM), "-p", String.valueOf(P_PARAM), "-q" ,train_file_name, model_file_name};
                //String argv[] = {"./libsvm-3.17/svm-train", "-s", "3", "-t", "2", "-c", "1024", "-q" ,"train.dat", "test.model"};
                if(KERNEL.equals("GAUSSIAN"))
                    {
                        argv[4] = "2";
                    }
                else if(KERNEL.equals("LINEAR"))
                    {
                        argv[4] = "0";
                    }

                Process p = Runtime.getRuntime().exec(argv);
                try
                    {
                        p.waitFor();
                    }
                catch(InterruptedException e)
                    {
		       
                    }
                String argv2[] = {"./libsvm-3.17/svm-predict", test_file_name, model_file_name, prediction_file_name};

                p = Runtime.getRuntime().exec(argv2);
                try
                    {
                        p.waitFor();
                    }
                catch(InterruptedException e)
                    {
  
                    }
                BufferedReader fin = new BufferedReader(new FileReader(prediction_file_name));
                double prediction = Double.parseDouble(fin.readLine());


                points.get(i).Prediction = prediction;

                //remove intermediary files
                String argv3[] = {"rm", train_file_name};
                p = Runtime.getRuntime().exec(argv3);
                try
                    {
                        p.waitFor();
                    }
                catch(InterruptedException e)
                    {
  
                    }
                argv3[1] = test_file_name;
                p = Runtime.getRuntime().exec(argv3);
                try
                    {
                        p.waitFor();
                    }
                catch(InterruptedException e)
                    {
  
                    }
                argv3[1] = model_file_name;
                p = Runtime.getRuntime().exec(argv3);
                try
                    {
                        p.waitFor();
                    }
                catch(InterruptedException e)
                    {
  
                    }

                argv3[1] = prediction_file_name;
                p = Runtime.getRuntime().exec(argv3);
                try
                    {
                        p.waitFor();
                    }
                catch(InterruptedException e)
                    {
  
                    }

                physiological_model.close();
                physiological_model_test.close();
                arima_models.close();
                arima_test.close();

                cur_point++;
                double percent = (((double)cur_point)/point_number) * 100;
                System.out.println(String.format("%.2f", percent) + "% complete");
		
            }
               
        printResults_p();
      	
      
    }}

