package BGPredict;


import java.util.Scanner;
import java.util.regex.Pattern;
//import BGPredict.ExtractData.SerializeExamples;
//import Classifiers.SVM.svm_predict;
//import Classifiers.SVM.svm_train;
import Diabetes.DataObjects.Reading;
import Diabetes.Diabetes;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;
import util.InputUtil;
/**
 * Main class for all processes of BGL prediction.
 * Inherit this class to gain access to the configuration of the BGL SVR Prediction System.
 * @author Matthew Wiley
 * @since 1-17-11
 */
public abstract class PredictionConfig extends Diabetes {

    public static boolean STATS = true;
    public static final String RAW = "raw";
    public static final String SMOOTH = "smooth";
    public static final String REALTIME = "realtime";
    public static boolean SCALE = true;
    public static int WINDOW = 1;
    public static String KERNEL = "GAUSSIAN";
    public static int LOOKAHEAD = 30;
    public static double C_PARAM = 2048;
    public static double P_PARAM = 0.0625;
    public static double G_PARAM = 0.5;
    public static String OUTPUT = "output.txt";
    protected static String DATASET = "input_data/dataset.dat";
    protected static boolean REAL_TIME = false;
    /**
     * Number of necessary previous points to consider a new point to be valid
     */
    protected static final int NUM_PREV_POINTS = 12;
    /**
     * If true, the output label for feature vectors should be set to the difference in BGL values, i.e. t+k - t_0.
     */
    protected static final boolean TAKE_DIFF = false;
    protected static final String EXPERIMENT_DIR = "../PredictExperiments";
    /**
     * Holds configuration information for the experiment
     */
    protected static final String EXPERIMENT_CONFIG = "config.txt";
    /**
     * Holds examples for the experiment with time stamps, comma separated
     */
    protected static final String DATA_FILE = "data.csv";
    /**
     * File extension used for svm files.
     */
    protected static final String SVM_FILE = "svm.dat";
    /**
     * Only needs to be done once per experiment, should be set to false after creation has been completed.
     */
    protected static final boolean CREATE_AR_DATA = false;
    /**
     * This includes the AR guess as a feature, may not help very well with raw data.
     */
    protected static final boolean SAVE_AR_GUESS = true;
    protected static final String R_FILE = "R.dat";
    protected static final boolean CLASSIFY_DIFF = false;
    protected static final String SCALE_FILE = "scaled_params";
    protected static final String SCALE_SUFFIX = ".scaled";

    /**
     * If true, attempts to adapt the SVM parameters.
     */
    protected static final boolean ADAPTIVE_PARAMTERS = true;
    public static void Configure(){
        Scanner scan = new Scanner(System.in);
        System.out.println("Number of weeks for the window to train on ("+ WINDOW +")");
        Pattern opts = Pattern.compile("^([0-9]*)$|^$");
        String choice = InputUtil.getInput(scan, opts);
        if(choice.equals("")){
         //WINDOW = 1;
        }else{
         WINDOW = Integer.parseInt(choice);
        }
        System.out.print("Scale features y/n (");
        if(SCALE){
         System.out.println("y)");
        }else{
         System.out.println("n)");
        }
        opts = Pattern.compile("^([yYnN])$|^$");
        choice = InputUtil.getInput(scan, opts).toUpperCase();
        if(choice.equals("")){
         //SCALE = true;
        }else if(choice.equals("Y")){
         SCALE = true;
        }else{
         SCALE = false;
        }
        System.out.println("Kernel Gaussian/Linear (" + KERNEL + ")");
        opts = Pattern.compile("^([gG][aA][uU][sS][sS][iI][aA][nN])$|^([lL][iI][nN][eE][aA][rR])$|^$");
        choice = InputUtil.getInput(scan, opts).toUpperCase();
        if(choice.equals("")){
         //KERNEL = "GAUSSIAN";
        }else if(choice.equals("GAUSSIAN")){
         KERNEL = "GAUSSIAN";
        }else{
         KERNEL = "LINEAR";
        }
        System.out.println("Prediction horizon in minutes, 5 minute intervals from 5-60 minutes: (" + LOOKAHEAD + ")");
        opts = Pattern.compile("^(5)$|^(10)$|^(15)$|^(20)$|^(25)$|^(30)$|^(35)$|^(40)$|^(45)$|^(50)$|^(55)$|^(60)$|^$");
        choice = InputUtil.getInput(scan, opts);
        if(choice.equals("")){
         //KERNEL = "GAUSSIAN";
	}else if(choice.equals("5")){
         LOOKAHEAD = 5;
	}else if(choice.equals("10")){
         LOOKAHEAD = 10;
	}else if(choice.equals("15")){
         LOOKAHEAD = 15;
	}else if(choice.equals("20")){
         LOOKAHEAD = 20;
	}else if(choice.equals("25")){
         LOOKAHEAD = 25;
        }else if(choice.equals("30")){
         LOOKAHEAD = 30;
	}else if(choice.equals("35")){
         LOOKAHEAD = 35;
	}else if(choice.equals("40")){
         LOOKAHEAD = 40;
	}else if(choice.equals("45")){
         LOOKAHEAD = 45;
	}else if(choice.equals("50")){
         LOOKAHEAD = 50;
	}else if(choice.equals("55")){
         LOOKAHEAD = 55;
        }else{
         LOOKAHEAD = 60;
        }
        
    }
    public PredictionConfig(int pid) {
        super(pid);
    }
    public PredictionConfig() {
        super(0);
    }
    protected double GenerateLabelValue(Reading t0, Reading label) {
        double ret = 0;
        if (TAKE_DIFF) {
            if (CLASSIFY_DIFF) {
                double diff = label.getValue() - t0.getValue();
                if (Math.abs(diff) < 10) { // if haven't changed by 10 points
                    ret = 0;
                } else if (diff >= 10) { // changed > +10
                    ret = 1;
                } else { // changed < -10
                    ret = 2;
                }
            } else {
                ret = label.getValue() - t0.getValue();
            }
        } else {
            ret = label.getValue();
        }
        return ret;
    }

    /*protected double RunSVM(String dir, String filetype, String train, String test, String[] args) throws FileNotFoundException, IOException {
        return RunSVM(dir, filetype, train, test, args, true, null);
    }
    protected List<Reading> predicts;

    protected double RunSVM(String dir, String filetype, String train, String test, String[] args, boolean silence, List<Reading> times) throws FileNotFoundException, IOException {
        return RunSVM(dir, filetype, train, test, args, silence, times, 0, -1);
    }

    protected double RunSVM(String dir, String filetype, String train, String test, String[] args, boolean silence, List<Reading> times, double a, int n) throws FileNotFoundException, IOException {
        return RunSVM(dir, filetype, train, test, args, silence, times, a, n, -1);
    }

    protected double RunSVM(String dir, String filetype, String train, String test, String[] args, boolean silence, List<Reading> times, double a, int n, double b) throws FileNotFoundException, IOException {
        System.out.println("Trying SVM with args: ");
        System.out.print("   ");
        for (String s : args) {
            System.out.print(s + " ");
        }
        if (n > 0 && a > 0) {
            System.out.print(" and c decay = " + a);
        }
        if (n > 0 && b > 0) {
            System.out.print(" and epsilon decay = " + b);
        }
        System.out.println("");
        String[] argv = new String[args.length + 2];
        for (int i = 0; i < args.length; ++i) {
            argv[i] = args[i];
        }
        PrintStream orig = System.out;
        PrintStream junk = new PrintStream("junk.txt");
        if (silence) {
            System.setOut(junk);
        }
        SerializeExamples se = new SerializeExamples(pid);
        se.ScaleSVMFile(dir, train, true, filetype);
        argv[args.length] = dir + train + SCALE_SUFFIX;
        argv[args.length + 1] = dir + filetype + "model";
        List<Double> weights = null;
        List<Double> e_weights = null;
        if (n > 0 && a > 0) {
            double C = -1;
            for (int i = 0; i < args.length; ++i) {
                if (args[i].equals("-c")) {
                    C = Double.parseDouble(args[i + 1]);
                }
            }
            weights = new ArrayList<Double>();
            for (int i = 1; i <= n; ++i) {
                weights.add(C * 2.0 * AscendingValue(a, i, n));
            }
        }
        if (n > 0 && b > 0) {
            double epsilon = -1;
            for (int i = 0; i < args.length; ++i) {
                if (args[i].equals("-p")) {
                    epsilon = Double.parseDouble(args[i + 1]);
                }
            }
            e_weights = new ArrayList<Double>();
            for (int i = 1; i <= n; ++i) {
                e_weights.add(epsilon * DescendingValue(b, i, n));
            }
        }
        svm_train.main(argv, null, weights, e_weights);
        se.ScaleSVMFile(dir, test, false, filetype);
        argv = new String[]{dir + test + SCALE_SUFFIX, dir + filetype + "model", dir + "output"};
        predicts = svm_predict.main(argv, times);
        System.setOut(orig);
        System.out.println(" RMSE: " + svm_predict.RMSE + "  NSV: " + svm_train.NSV);
        return svm_predict.RMSE;
    }
*/
    private double AscendingValue(double a, int i, int n) {
        double exp_part = (a) - ((2 * a * i) / ((double) n));
        double ret_val = 1.0 / (1.0 + Math.exp(exp_part));
        return ret_val;
    }

    private double DescendingValue(double b, int i, int n) {
        double exp_part = (b) - ((2 * b * i) / ((double) n));
        double ret_val = (1.0 + Math.exp(exp_part)) / 2.0;
        return ret_val;
    }

	public static void SetLookahead(int t)
	{
		LOOKAHEAD = t;
	}

	public static void SetWindow(int w)
	{
		WINDOW = w;
	}
	public static void SetKernel(String k)
	{
		KERNEL = k;
	}
	public static void SetOutput(String o)
	{
		OUTPUT = o;
	}
	public static void SetScale(boolean s)
	{
		SCALE = s;
	}

	public static void SetDataset(String d)
	{
		DATASET = d;
	}
	
	public static void SetC(double c)
	{
		C_PARAM = c;
	}

	public static void SetP(double p)
	{
		P_PARAM = p;
	}

	public static void SetG(double g)
	{
		G_PARAM = g;
	}

	
}
