
package Diabetes.DataObjects;

import java.util.GregorianCalendar;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

/**
 * Contains the structures used for loading the daily schedule for a patient.
 * @author Matthew Wiley
 * @version 1.0
 * @since Jun 8, 2010
 */
public class DailySchedule {

    /**
     * Constructor - initializes all the TreeMaps
     */
    public DailySchedule(){
        daily_schedule = new TreeMap<Integer, DailyScheduleInfo>();
        daily_work = new TreeMap<Integer, DailyScheduleInfo>();
        daily_snack = new Vector<Map<Integer, DailyScheduleInfo>>();
    }
    /**
     *
     * @param day
     * @param start
     * @param end
     */
    public void put_work(int day, GregorianCalendar start, GregorianCalendar end){
        daily_work.put(day, new DailyScheduleInfo(start, end));
    }
    /**
     *
     * @param day
     * @param start
     * @param end
     * @param breakfast
     * @param lunch
     * @param dinner
     */
    public void put_daily_schedule(int day, GregorianCalendar start, GregorianCalendar end, GregorianCalendar breakfast, GregorianCalendar lunch, GregorianCalendar dinner){
        daily_schedule.put(day, new DailyScheduleInfo(start, end, breakfast, lunch, dinner));
    }
    /**
     *
     * @param day
     * @param start
     */
    public void put_daily_snack(int day, GregorianCalendar start){
        boolean added = false;
        for(Map<Integer, DailyScheduleInfo> m : daily_snack){
            if(m.containsKey(day)){
                continue;
            } else {
                added = true;
                m.put(day, new DailyScheduleInfo(start, null));
            }
        }
        if(!added){
            daily_snack.add(new TreeMap<Integer, DailyScheduleInfo>());
            daily_snack.lastElement().put(day, new DailyScheduleInfo(start, null));
        }
    }

    /**
     *
     * @return Map of the daily schedule <int, DailyScheduleInfo> where int represents the day of week (1-7)
     */
    public Map<Integer, DailyScheduleInfo> getDaily_schedule() {
        return daily_schedule;
    }

    /**
     *
     * @return Vector of Map of the daily work schedule <int, DailyScheduleInfo> where int represents the day of week (1-7)
     */
    public Vector<Map<Integer, DailyScheduleInfo>> getDaily_snack() {
        return daily_snack;
    }

    /**
     *
     * @return Map of the daily work schedule <int, DailyScheduleInfo> where int represents the day of week (1-7)
     */
    public Map<Integer, DailyScheduleInfo> getDaily_work() {
        return daily_work;
    }
    
    private Map<Integer, DailyScheduleInfo> daily_schedule; // start/end used for to sleep / awake
    private Map<Integer, DailyScheduleInfo> daily_work;  // start/end used for to work / from work
    private Vector<Map<Integer, DailyScheduleInfo> > daily_snack;

}
