package Diabetes;

import Diabetes.DataObjects.Reading;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import util.DBUtil;
import util.DateUtil;

/**
 * Base class, which is extended by all 3 systems (SVR, Classifier, Preprocessing).
 * Contains common methods for all classes dealing with diabetes.
 * @author Matthew Wiley
 * @since 3-4-11
 */
public class Diabetes {

    /**
     * Interval (in minutes) between CGMS readings
     */
    protected static final int CGMS_INTERVAL = 5;
    
    public Diabetes(int pid){
        this.pid = pid;
    }
    protected int pid;
    protected List<Reading> getCGMS() {
        return getCGMS(null, null);
    }

    protected List<Reading> getCGMS(GregorianCalendar start, GregorianCalendar end) {
        return getCGMS(start, end, "cgms WHERE");
    }

    protected List<Reading> getCGMS(GregorianCalendar start, GregorianCalendar end, String table_with_where) {
        List<Reading> points = new ArrayList<Reading>();
        Connection conn = DBUtil.createConnection();
        try {
            Statement stmt = conn.createStatement();
            String query = null;
            if (start == null || end == null) {
                query = "SELECT * FROM " + table_with_where + " PID = " + pid + " ORDER BY ts";
            } else {
                query = "SELECT * FROM " + table_with_where + " PID = " + pid + " AND TS >= " + DBUtil.createQueryTS(DateUtil.makeTS(start)) + " AND TS <= "
                        + DBUtil.createQueryTS(DateUtil.makeTS(end)) + " ORDER BY ts";
            }
            ResultSet rs = DBUtil.QueryDB(query);
            while (rs.next()) {
                points.add(new Reading(DateUtil.createGC(rs.getString("ts")), rs.getDouble("val")));
            }
            conn.close();
        } catch (SQLException ex) {
            Logger.getLogger(DBUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
        return points;
    }
}
