#!/usr/bin/env python
# This gridregression.py is modified from grid.py in LIBSVM 3.17 version
__all__ = ['find_parameters']

import os, sys, traceback, getpass, time, re
from threading import Thread
from subprocess import *

if sys.version_info[0] < 3:
	from Queue import Queue
else:
	from queue import Queue

telnet_workers = []
ssh_workers = ["bin00001.cs.ohio.edu","bin00010.cs.ohio.edu","bin00011.cs.ohio.edu","bin00100.cs.ohio.edu"\
              ,"bin00101.cs.ohio.edu","bin00110.cs.ohio.edu","bin00111.cs.ohio.edu","bin01000.cs.ohio.edu"\
              ,"bin01001.cs.ohio.edu","bin01010.cs.ohio.edu","bin01011.cs.ohio.edu","bin01100.cs.ohio.edu"\
              ,"bin01101.cs.ohio.edu","bin01110.cs.ohio.edu","bin01111.cs.ohio.edu","bin10000.cs.ohio.edu"\
              ,"bin10001.cs.ohio.edu","bin10010.cs.ohio.edu","bin10011.cs.ohio.edu","bin10100.cs.ohio.edu"\
              
              ,"sp-001.cs.ohio.edu","sp-002.cs.ohio.edu","sp-003.cs.ohio.edu","sp-004.cs.ohio.edu","sp-005.cs.ohio.edu"\
              ,"sp-006.cs.ohio.edu","sp-007.cs.ohio.edu","sp-008.cs.ohio.edu","sp-009.cs.ohio.edu","sp-010.cs.ohio.edu"\
              ,"sp-011.cs.ohio.edu","sp-012.cs.ohio.edu","sp-013.cs.ohio.edu","sp-014.cs.ohio.edu","sp-015.cs.ohio.edu"\
              ,"sp-016.cs.ohio.edu","sp-017.cs.ohio.edu","sp-018.cs.ohio.edu","sp-019.cs.ohio.edu","sp-020.cs.ohio.edu"\
       ]        
#ssh_workers = ["99.54.97.118"]
#ssh_workers = []
nr_local_worker = 0

class GridOption:
	def __init__(self, options):
		dirname = os.path.dirname(__file__)
		if sys.platform != 'win32':
			self.svmtrain_pathname = os.path.join(dirname, '../svm-train')
			self.svmpredict_pathname = os.path.join(dirname, '../svm-predict')
			self.gnuplot_pathname = '/usr/bin/gnuplot'
		else:
			# example for windows
			self.svmtrain_pathname = os.path.join(dirname, r'..\windows\svm-train.exe')
			# svmtrain_pathname = r'c:\Program Files\libsvm\windows\svm-train.exe'
			self.gnuplot_pathname = r'c:\tmp\gnuplot\binary\pgnuplot.exe'
		self.fold = 5
		self.c_begin, self.c_end, self.c_step = 8,  12,  1
		self.g_begin, self.g_end, self.g_step =  -3, -8, -1
		self.p_begin, self.p_end, self.p_step = -8,  -1,  1
		self.points_begin, self.points_end, self.points_step = 1,196,1
		self.grid_with_c, self.grid_with_g, self.grid_with_p = True, True, True
		#self.dataset_title = "grid"
		#self.out_pathname = '{0}.out'.format(self.dataset_title)
		#self.png_pathname = '{0}.png'.format(self.dataset_title)
		self.pass_through_string = ' '
		self.resume_pathname = None
		self.parse_options(options)

	def parse_options(self, options):
		if type(options) == str:
			options = options.split()
		i = 0
		pass_through_options = []
		
		while i < len(options):
			if options[i] == '-log2c':
				i = i + 1
				if options[i] == 'null':
					self.grid_with_c = False
				else:
					self.c_begin, self.c_end, self.c_step = map(float,options[i].split(','))
			elif options[i] == '-log2g':
				i = i + 1
				if options[i] == 'null':
					self.grid_with_g = False
				else:
					self.g_begin, self.g_end, self.g_step = map(float,options[i].split(','))
			elif options[i] == '-log2p' :
				i = i + 1
				if options[i] == 'null':
					self.grid_with_p = False	
				else:
					self.p_begin, self.p_end, self.p_step = map(float,options[i].split(','))
			elif options[i] == '-v':
				i = i + 1
				self.fold = options[i]
			elif options[i] in ('-c','-g','-p'):
				raise ValueError('Use -log2c , -log2g and -log2p.')
			elif options[i] == '-svmtrain':
				i = i + 1
				self.svmtrain_pathname = options[i]
			elif options[i] == '-svmpredict':
				i = i + 1
				self.svmpredict_pathname = options[i]
			elif options[i] == '-points':
			 i = i + 1
			 self.points_begin, self.points_end, self.points_step = map(int,options[i].split(','))
			elif options[i] == '-gnuplot':
				i = i + 1
				if options[i] == 'null':
					self.gnuplot_pathname = None
				else:	
					self.gnuplot_pathname = options[i]
			elif options[i] == '-out':
				i = i + 1
				if options[i] == 'null':
					self.out_pathname = None
				else:
					self.out_pathname = options[i]
			elif options[i] == '-png':
				i = i + 1
				self.png_pathname = options[i]
			elif options[i] == '-resume':
				if i == (len(options)-1) or options[i+1].startswith('-'):
					self.resume_pathname = self.dataset_title + '.out'
				else:
					i = i + 1
					self.resume_pathname = options[i]
			else:
				pass_through_options.append(options[i])
			i = i + 1

		self.pass_through_string = ' '.join(pass_through_options)
		#if not os.path.exists(self.svmtrain_pathname):
		#	raise IOError('svm-train executable not found')
		#if not os.path.exists(self.dataset_pathname):
		#	raise IOError('dataset not found')
		#if self.resume_pathname and not os.path.exists(self.resume_pathname):
		#	raise IOError('file for resumption not found')
		#if not self.grid_with_c and not self.grid_with_g and not self.grid_with_p:
		#	raise ValueError('-log2c , -log2g and -log2p should not be null simultaneously')
		# gridregression default did not support gnuplot
		self.gnuplot_pathname = None
		if self.gnuplot_pathname and not os.path.exists(self.gnuplot_pathname):
			sys.stderr.write('gnuplot executable not found\n')
			self.gnuplot_pathname = None

def redraw(db,best_param,gnuplot,options,tofile=False):
	if len(db) == 0: return
	begin_level = round(max(x[2] for x in db)) - 3
	step_size = 0.5

	best_log2c,best_log2g,best_rate = best_param

	# if newly obtained c, g, or cv values are the same,
	# then stop redrawing the contour.
	if all(x[0] == db[0][0]  for x in db): return
	if all(x[1] == db[0][1]  for x in db): return
	if all(x[2] == db[0][2]  for x in db): return

	if tofile:
		gnuplot.write(b"set term png transparent small linewidth 2 medium enhanced\n")
		gnuplot.write("set output \"{0}\"\n".format(options.png_pathname.replace('\\','\\\\')).encode())
		#gnuplot.write(b"set term postscript color solid\n")
		#gnuplot.write("set output \"{0}.ps\"\n".format(options.dataset_title).encode().encode())
	elif sys.platform == 'win32':
		gnuplot.write(b"set term windows\n")
	else:
		gnuplot.write( b"set term x11\n")
	gnuplot.write(b"set xlabel \"log2(C)\"\n")
	gnuplot.write(b"set ylabel \"log2(gamma)\"\n")
	gnuplot.write("set xrange [{0}:{1}]\n".format(options.c_begin,options.c_end).encode())
	gnuplot.write("set yrange [{0}:{1}]\n".format(options.g_begin,options.g_end).encode())
	gnuplot.write(b"set contour\n")
	gnuplot.write("set cntrparam levels incremental {0},{1},100\n".format(begin_level,step_size).encode())
	gnuplot.write(b"unset surface\n")
	gnuplot.write(b"unset ztics\n")
	gnuplot.write(b"set view 0,0\n")
	gnuplot.write("set title \"{0}\"\n".format(options.dataset_title).encode())
	gnuplot.write(b"unset label\n")
	gnuplot.write("set label \"Best log2(C) = {0}  log2(gamma) = {1}  accuracy = {2}%\" \
				  at screen 0.5,0.85 center\n". \
				  format(best_log2c, best_log2g, best_rate).encode())
	gnuplot.write("set label \"C = {0}  gamma = {1}\""
				  " at screen 0.5,0.8 center\n".format(2**best_log2c, 2**best_log2g).encode())
	gnuplot.write(b"set key at screen 0.9,0.9\n")
	gnuplot.write(b"splot \"-\" with lines\n")
	
	db.sort(key = lambda x:(x[0], -x[1]))

	prevc = db[0][0]
	for line in db:
		if prevc != line[0]:
			gnuplot.write(b"\n")
			prevc = line[0]
		gnuplot.write("{0[0]} {0[1]} {0[2]}\n".format(line).encode())
	gnuplot.write(b"e\n")
	gnuplot.write(b"\n") # force gnuplot back to prompt when term set failure
	gnuplot.flush()


def calculate_jobs(options):
	
	def range_f(begin,end,step):
		# like range, but works on non-integer too
		seq = []
		while True:
			if step > 0 and begin > end: break
			if step < 0 and begin < end: break
			seq.append(begin)
			begin = begin + step
		return seq
	
	def permute_sequence(seq):
		n = len(seq)
		if n <= 1: return seq
	
		mid = int(n/2)
		left = permute_sequence(seq[:mid])
		right = permute_sequence(seq[mid+1:])
	
		ret = [seq[mid]]
		while left or right:
			if left: ret.append(left.pop(0))
			if right: ret.append(right.pop(0))
			
		return ret	

	
	points_seq = permute_sequence(range_f(options.points_begin,options.points_end,options.points_step))
	
	
	
	
	nr_points = len(points_seq)

	jobs = []

	for i in range(0,nr_points):
		line=[]
		line.append(points_seq[i])
		jobs.append(line)
	resumed_jobs = {}
	
	if options.resume_pathname is None:
		return jobs, resumed_jobs

	for line in open(options.resume_pathname, 'r'):
		line = line.strip()
		rst = re.findall(r'mse=([0-9.]+)',line)
		if not rst: 
			continue
		mse = float(rst[0])

		c, g, p = None, None, None 
		rst = re.findall(r'log2c=([0-9.-]+)',line)
		if rst: 
			c = float(rst[0])
		rst = re.findall(r'log2g=([0-9.-]+)',line)
		if rst: 
			g = float(rst[0])
		rst = re.findall(r'log2p=([0-9,-]+)',line)
		if rst:
			p = float(rst[0])

		resumed_jobs[(c,g,p)] = mse

	return jobs, resumed_jobs

	
class WorkerStopToken:  # used to notify the worker to stop or if a worker is dead
	pass

class Worker(Thread):
	def __init__(self,name,job_queue,result_queue,options):
		Thread.__init__(self)
		self.name = name
		self.job_queue = job_queue
		self.result_queue = result_queue
		self.options = options
		
	def run(self):
		while True:
			(point) = self.job_queue.get()
			if point is WorkerStopToken:
				self.job_queue.put((point))
				#print('worker {0} stop.'.format(self.name))
				break
			try:
				
				mse = self.run_one(point)
				if mse is None: raise RuntimeError('get no mse')
			except:
				# we failed, let others do that and we just quit
			
				#traceback.print_exception(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
				
				self.job_queue.put((point))
				#sys.stderr.write('worker {0} quit.\n'.format(self.name))
				#sys.stderr.write('command: \'' + self.cmdline + "\'\n")
				return
				continue
			else:
				self.result_queue.put((self.name,point,mse))

	def get_cmd(self,point):
		options=self.options
		cmdline = './gridsearch.py -log2c 8,12,1 -log2g -3,-8,-1 -log2p 0,-8,-1 -point {0} -j'.format(point)
		return cmdline
		
class LocalWorker(Worker):
	def run_one(self,point):
		cmdline = self.get_cmd(point)
		#print(cmdline)
		result = Popen(cmdline,shell=True,stdout=PIPE,stderr=PIPE,stdin=PIPE).stdout
		for line in result.readlines():
			if str(line).find('test error = ') != -1:
				
				return float(line.split()[-1])

class SSHWorker(Worker):
	def __init__(self,name,job_queue,result_queue,host,options):
		Worker.__init__(self,name,job_queue,result_queue,options)
		self.host = host
		#self.cwd = os.getcwd()
		self.cwd = "~/git/prediction"
		
	def run_one(self,point):
	 #username = ""
		if self.host == "99.54.97.118": 
		 username = "nigel" 
		else:
		 username = "nstruble"
		cmdline = 'ssh -l ' + username + ' -x -t -t {0} "cd {1}; {2}"'.format\
			(self.host,self.cwd,self.get_cmd(point))
		#print(cmdline)
		self.cmdline = cmdline
		result = Popen(cmdline,shell=True,stdout=PIPE,stderr=PIPE,stdin=PIPE).stdout
		for line in result.readlines():
			if str(line).find('test error = ') != -1:
				#print "parsing line: " + str(line)
				#print "returning " + line.split()[-2]
				return float(line.split()[-1])

class TelnetWorker(Worker):
	def __init__(self,name,job_queue,result_queue,host,username,password,options):
		Worker.__init__(self,name,job_queue,result_queue,options)
		self.host = host
		self.username = username
		self.password = password		
	def run(self):
		import telnetlib
		self.tn = tn = telnetlib.Telnet(self.host)
		tn.read_until('login: ')
		tn.write(self.username + '\n')
		tn.read_until('Password: ')
		tn.write(self.password + '\n')

		# XXX: how to know whether login is successful?
		tn.read_until(self.username)
		# 
		print('login ok', self.host)
		tn.write('cd '+os.getcwd()+'\n')
		Worker.run(self)
		tn.write('exit\n')			   
	def run_one(self,c,g,p):
		cmdline = self.get_cmd(c,g,p)
		result = self.tn.write(cmdline+'\n')
		(idx,matchm,output) = self.tn.expect(['Cross.*\n'])
		for line in output.split('\n'):
			if str(line).find('test error = ') != -1:
				return float(line.split()[-2])
			
def find_parameters(options=''):
	
	def update_param(c,g,p,mse,best_c,best_g,best_p,best_mse,worker,resumed):
		if ( mse < best_mse ):
			best_mse,best_c,best_g,best_p = mse,c,g,p
		stdout_str = '[{0}] {1} {2} (best '.format\
			(worker,' '.join(str(x) for x in [c,g,p] if x is not None),mse)
		output_str = ''
		if c != None:
			stdout_str += 'c={0}, '.format(2.0**best_c)
			output_str += 'log2c={0} '.format(c)
		if g != None:
			stdout_str += 'g={0}, '.format(2.0**best_g)
			output_str += 'log2g={0} '.format(g)
		if p != None:
			stdout_str += 'p={0}, '.format(2.0**best_p)
			output_str += 'log2p={0} '.format(p)
		stdout_str += 'mse={0})'.format(best_mse)
		print(stdout_str)
		if options.out_pathname and not resumed:
			output_str += 'mse={0}\n'.format(mse)
			result_file.write(output_str)
			result_file.flush()
		
		return best_c,best_g,best_p,best_mse
		
	options = GridOption(options);

	if options.gnuplot_pathname:
		gnuplot = Popen(options.gnuplot_pathname,stdin = PIPE,stdout=PIPE,stderr=PIPE).stdin
	else:
		gnuplot = None
		
	# put jobs in queue

	jobs,resumed_jobs = calculate_jobs(options)
	job_queue = Queue(0)
	result_queue = Queue(0)

	for (point) in resumed_jobs:
		result_queue.put(('resumed',point,resumed_jobs[(point)]))

	for line in jobs:
		for (point) in line:
			if (point) not in resumed_jobs:
				job_queue.put((point))

	# hack the queue to become a stack --
	# this is important when some thread
	# failed and re-put a job. It we still
	# use FIFO, the job will be put
	# into the end of the queue, and the graph
	# will only be updated in the end
 
	job_queue._put = job_queue.queue.appendleft

	# fire telnet workers

	if telnet_workers:
		nr_telnet_worker = len(telnet_workers)
		username = getpass.getuser()
		password = getpass.getpass()
		for host in telnet_workers:
			worker = TelnetWorker(host,job_queue,result_queue,
					 host,username,password,options)
			worker.start()

	# fire ssh workers

	if ssh_workers:
		for host in ssh_workers:
		 if host == "99.54.97.118": 
		  remote_jobs = 10
		 elif host.find("sp") == 0:
		 	remote_jobs = 8
		 else:
		  remote_jobs = 4
		 remote_jobs = 1
		 for i in range(0,remote_jobs):
			 worker = SSHWorker(host,job_queue,result_queue,host,options)
			 worker.start()

	# fire local workers

	for i in range(nr_local_worker):
		worker = LocalWorker('local',job_queue,result_queue,options)
		worker.start()

	# gather results

	done_jobs = {}

	


	db = []
	best_mse = float('+inf')
	best_c,best_g,best_p = None,None,None  

	for (c,g,p) in resumed_jobs:
		mse = resumed_jobs[(c,g,p)]
		best_c,best_g,best_p,best_mse = update_param(c,g,p,mse,best_c,best_g,best_p,best_mse,'resumed',True)
	rmse = 0
	for line in jobs:
		for (point) in line:
			while (point) not in done_jobs:
				(worker,point1,mse1) = result_queue.get()
				done_jobs[(point1)] = mse1
				if (point1) not in resumed_jobs:
					sys.stdout.write(str(len(jobs) - job_queue.qsize() - len(done_jobs)) + " ")
					#best_c,best_g,best_p,best_mse = update_param(c1,g1,p1,mse1,best_c,best_g,best_p,best_mse,worker,False)
					sys.stdout.write("test " + str(point1) + " error = " + str(mse1) + "\n")
					rmse += mse1 * mse1
			db.append((point,done_jobs[(point)]))
		if gnuplot and options.grid_with_c and options.grid_with_g:
			redraw(db,[best_c, best_g, best_rate],gnuplot,options)
			redraw(db,[best_c, best_g, best_rate],gnuplot,options,True)

	job_queue.put((WorkerStopToken))
	rmse = (rmse/len(jobs))**0.5
	print("rmse = " + str(rmse))

	return rmse


if __name__ == '__main__':

	def exit_with_help():
		print("""\
Usage: distributed.py [grid_options]

grid_options :
-points {begin,end,step} : set the range of the test points (default 1, 196, 1)
    begin,end,step -- poiont_range = {begin,...,begin+k*step,...,end}

""")
		sys.exit(1)
	
	if len(sys.argv) < 2:
		exit_with_help()
	options = sys.argv[1:]
	try:
		find_parameters(options)
	except (IOError,ValueError) as e:
		sys.stderr.write(str(e) + '\n')
		sys.stderr.write('Try "gridregression.py" for more information.\n')
		sys.exit(1)
