package util;
import java.io.Console;
import java.io.ObjectInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import Diabetes.DataObjects.DailySchedule;
import Diabetes.DataObjects.Reading;
import Diabetes.DataObjects.BolusReading;
import Diabetes.DataObjects.BasalReading;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import dbpatient.OraclePatient;
import dbpatient.TSEvent;
import dbpatient.TSESleep;
import java.util.Vector;
/**
 *
 * @author Matthew Wiley
 * @since 4-14-10
 */
public class DBUtil {

    /**
     * Contains the location of the SQLite file.  This needs to be configured to run the experiments!
     */
    private static final String SQLITE_LOCATION = "../diabetes.db";
//    private static final String SQLITE_LOCATION = "C:\\Users\\Matt\\Documents\\diabetes.db";
//    private static final String SQLITE_LOCATION = "/home/mwiley/ai_grad/diabetes.db";
    public static String DB_USER = null;
    public static String DB_PASS = null;
    //public static String DB_CLASS = "org.sqlite.JDBC";
    //public static String DB_LINK = "jdbc:sqlite:" + SQLITE_LOCATION;
    
    public static OraclePatient OP = null;
    
    public static String DB_CLASS = "oracle.jdbc.driver.OracleDriver";
    public static String DB_LINK = "jdbc:oracle:oci:@";
    public static Connection conn = null;
    public static Statement stmt = null;

    /**
     * Creates a database connection. NOTE, the caller must close the connection to free memory.
     * @return a database connection
     */
    public static Connection createConnection() {
        try {
            if(DB_USER == null || DB_PASS == null){
				          Console cons = System.console();
				          DB_USER = new String(cons.readLine("%s", "Username: "));
				          cons.flush();
				  								DB_PASS = new String(cons.readPassword("%s", "Password: "));
				  						}
            Connection conn = null;
            Class.forName(DB_CLASS);
            //conn = DriverManager.getConnection(DB_LINK);
            conn = DriverManager.getConnection(DB_LINK, DB_USER, DB_PASS);
            return conn;
        } catch (SQLException ex) {
            Logger.getLogger(DBUtil.class.getName()).log(Level.SEVERE, null, ex);
        } catch (ClassNotFoundException ex) {
            Logger.getLogger(DBUtil.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }
    
    public static void read_op(String ser) throws IOException, ClassNotFoundException {
        
        ObjectInputStream is = new ObjectInputStream(new FileInputStream(ser));
        OP = (OraclePatient) is.readObject();
        is.close();
    }
    public static List<Reading> op_getCGMS(GregorianCalendar start, GregorianCalendar end){
        List<Reading> points = new ArrayList<Reading>();
		      if (start == null || end == null) {
		          for(int i = 0; i < OP.getGlucoseLevels().size(); ++i){
		           points.add(new Reading(DateUtil.createEventGC(OP.getGlucoseLevels().get(i).getTS()), OP.getGlucoseLevels().get(i).value_));
		          }
		      } else {

		          for(int i = DBUtil.get_index(start,OP.getGlucoseLevels()); i < OP.getGlucoseLevels().size(); ++i){
		           GregorianCalendar gc = DateUtil.createEventGC(OP.getGlucoseLevels().get(i).getTS());
		           if(gc.before(start)){
		            continue;
		           }
		           if(gc.after(end)){
		            break;
		           }
		           points.add(new Reading(DateUtil.createEventGC(OP.getGlucoseLevels().get(i).getTS()), OP.getGlucoseLevels().get(i).value_));
		          }
		      }

        return points;
    }
    public static Reading op_nextCGMS(GregorianCalendar start){
        Reading point = null;
		      if (start == null) {
		          point = new Reading(DateUtil.createEventGC(OP.getGlucoseLevels().get(0).getTS()), OP.getGlucoseLevels().get(0).value_);
		      } else {
		          for(int i = DBUtil.get_index(start,OP.getGlucoseLevels()); i < OP.getGlucoseLevels().size(); ++i){
									     GregorianCalendar gc = DateUtil.createEventGC(OP.getGlucoseLevels().get(i).getTS());
									     if(gc.before(start) || gc.equals(start)){
									      continue;
									     }
									     point = new Reading(DateUtil.createEventGC(OP.getGlucoseLevels().get(i).getTS()), OP.getGlucoseLevels().get(i).value_);
									     break;
									    }
		      }
            
        return point;
    }
    public static List<Reading> op_getFingersticks(GregorianCalendar start, GregorianCalendar end){
        List<Reading> points = new ArrayList<Reading>();
		      if (start == null || end == null) {
		          for(int i = 0; i < OP.getFingerSticks().size(); ++i){
		           points.add(new Reading(DateUtil.createEventGC(OP.getFingerSticks().get(i).getTS()), OP.getFingerSticks().get(i).value_));
		          }
		      } else {

		          for(int i = DBUtil.get_index(start,OP.getFingerSticks()); i < OP.getFingerSticks().size(); ++i){
		           GregorianCalendar gc = DateUtil.createEventGC(OP.getFingerSticks().get(i).getTS());
		           if(gc.before(start)){
		            continue;
		           }
		           if(gc.after(end)){
		            break;
		           }
		           points.add(new Reading(DateUtil.createEventGC(OP.getFingerSticks().get(i).getTS()), OP.getFingerSticks().get(i).value_));
		          }
		      }

        return points;
    }
    public static List<Reading> op_getMeals(GregorianCalendar start, GregorianCalendar end){
        List<Reading> points = new ArrayList<Reading>();
		      if (start == null || end == null) {
		          for(int i = 0; i < OP.getMeals().size(); ++i){
		           points.add(new Reading(DateUtil.createEventGC(OP.getMeals().get(i).getTS()), OP.getMeals().get(i).carbs_));
		          }
		      } else {
            int index = DBUtil.get_index(start,OP.getMeals());
            //System.out.println("index = " + index);
		          for(int i = index; i < OP.getMeals().size(); ++i){
		          //for(int i = 0; i < OP.getMeals().size(); ++i){
		           GregorianCalendar gc = DateUtil.createEventGC(OP.getMeals().get(i).getTS());
		           if(gc.before(start)){
		            //System.out.println("i is before ");
		            continue;
		           }
		           if(gc.after(end)){
		            //System.out.println("i is after ");
		            break;
		           }
		           //System.out.println("i = " + i);
		           points.add(new Reading(DateUtil.createEventGC(OP.getMeals().get(i).getTS()), OP.getMeals().get(i).carbs_));
		          }
		      }

        return points;
    }
    public static List<BolusReading> op_getBoluses(GregorianCalendar start, GregorianCalendar end){
        List<BolusReading> points = new ArrayList<BolusReading>();
		      if (start == null || end == null) {
		          for(int i = 0; i < OP.getBoluses().size(); ++i){
		           points.add(new BolusReading(DateUtil.createEventGC(OP.getBoluses().get(i).getTS()),OP.getBoluses().get(i).getType(),OP.getBoluses().get(i).dose_,OP.getBoluses().get(i).dose_,OP.getBoluses().get(i).dose_));
		          }
		      } else {

		          for(int i = DBUtil.get_index(start,OP.getBoluses()); i < OP.getBoluses().size(); ++i){
		           GregorianCalendar gc = DateUtil.createEventGC(OP.getBoluses().get(i).getTS());
		           if(gc.before(start)){
		            continue;
		           }
		           if(gc.after(end)){
		            break;
		           }
		           points.add(new BolusReading(DateUtil.createEventGC(OP.getBoluses().get(i).getTS()),OP.getBoluses().get(i).getType(),OP.getBoluses().get(i).dose_,OP.getBoluses().get(i).dose_,OP.getBoluses().get(i).dose_));
		          }
		      }

        return points;
    }
    public static List<BasalReading> op_getBasals(GregorianCalendar start, GregorianCalendar end){
        List<BasalReading> points = new ArrayList<BasalReading>();
		      if (start == null || end == null) {
		          for(int i = 0; i < OP.getBasalValues().size()-1; ++i){
		           if(OP.getBasalValues().size() == i-1){
		            points.add(new BasalReading(OP.getBasalValues().get(i).value_,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS()),DateUtil.createEventGC(OP.getGlucoseLevels().get(OP.getGlucoseLevels().size()-1).getTS())));
		           }else{
		            points.add(new BasalReading(OP.getBasalValues().get(i).value_,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS()),DateUtil.createEventGC(OP.getBasalValues().get(i+1).getTS())));
		           }
		          }
		      } else {
            GregorianCalendar before_gc = DateUtil.createEventGC(OP.getBasalValues().get(0).getTS());
		          for(int i = 0; i < OP.getBasalValues().size(); ++i){
		           GregorianCalendar gc = DateUtil.createEventGC(OP.getBasalValues().get(i).getTS());
		           
		           if(gc.before(start)){
		            before_gc = DateUtil.createEventGC(OP.getBasalValues().get(i).getTS());
		            continue;
		           }
		           if(gc.after(end)){
		            break;
		           }
		           if(OP.getBasalValues().size() == i-1){//last basal reading
		            points.add(new BasalReading(OP.getBasalValues().get(i).value_,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS()),DateUtil.createEventGC(OP.getGlucoseLevels().get(OP.getGlucoseLevels().size()-1).getTS())));
		           }else{
		            if(before_gc != null){//first basal reading
		             if(i > 0){
		              points.add(new BasalReading(OP.getBasalValues().get(i-1).value_,before_gc,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS())));
		             }else{
		              points.add(new BasalReading(OP.getBasalValues().get(0).value_,before_gc,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS())));
		             }
		             if(OP.getBasalValues().size() == i-1){//first basal reading is also last basal reading
	               points.add(new BasalReading(OP.getBasalValues().get(i).value_,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS()),DateUtil.createEventGC(OP.getGlucoseLevels().get(OP.getGlucoseLevels().size()-1).getTS())));
		             }else{//first basal reading is not also last basal reading
		              points.add(new BasalReading(OP.getBasalValues().get(i).value_,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS()),DateUtil.createEventGC(OP.getBasalValues().get(i+1).getTS())));
		             }
		             before_gc = null;
		            }else{//normal basal reading
		             points.add(new BasalReading(OP.getBasalValues().get(i).value_,DateUtil.createEventGC(OP.getBasalValues().get(i).getTS()),DateUtil.createEventGC(OP.getBasalValues().get(i+1).getTS())));
		            }
		           }
		          }
		      }

        return points;
    }
    public static boolean op_is_asleep(GregorianCalendar start){
        int to_bed_index = DBUtil.get_sleep_index(start,OP.getSleep());
        if(DateUtil.createEventGC(OP.getSleep().get(to_bed_index).getEndTS()).before(start)){
         if(OP.getSleep().size() <= to_bed_index+1 || DateUtil.createEventGC(OP.getSleep().get(to_bed_index+1).getTS()).after(start)){
          return true;
          //System.out.println("ts " + DateUtil.makeTS(start) + " is asleep");
          //System.out.println("went to bed at " + OP.getSleep().get(to_bed_index).getEndTS());
          //System.out.println("will wake up at " + OP.getSleep().get(to_bed_index+1).getTS() + "\n");
         }
        }
        return false;
    }
    public static int op_time_asleep(GregorianCalendar start){
        if(!op_is_asleep(start)){
         return 0;
        }
        int to_bed_index = DBUtil.get_sleep_index(start,OP.getSleep());
        int diff = (int)(start.getTime().getTime()/60000) - (int)(DateUtil.createEventGC(OP.getSleep().get(to_bed_index).getEndTS()).getTime().getTime()/60000);
        //System.out.println("ts " + DateUtil.makeTS(start) + " is asleep");
        //System.out.println("went to bed at " + OP.getSleep().get(to_bed_index).getEndTS());
        //System.out.println("difference = " + diff);
        //return (int)DateUtil.createEventGC(OP.getSleep().get(to_bed_index).getEndTS()).getTime() - (int)DateUtil.createEventGC(OP.getSleep().get(to_bed_index+1).getTS()).getTime();
        return diff;
    }
    private static int get_index(GregorianCalendar gc, Vector<? extends TSEvent> event){
        int imax = event.size()-1;
        int imin = 0;
        while(imax >= imin){
          int imid = (imax + imin) / 2;
          GregorianCalendar gce = DateUtil.createEventGC(event.get(imid).getTS());
          if(gce.before(gc)){
           imin = imid + 1;
          }else if(gce.after(gc)){
           imax = imid - 1;
          }else{
           return imid;
          }
          
        }
        return (imax < 0) ? 0 : imax;
    }
    private static int get_sleep_index(GregorianCalendar gc, Vector<TSESleep> event){
        int imax = event.size()-1;
        int imin = 0;
        while(imax >= imin){
          int imid = (imax + imin) / 2;
          GregorianCalendar gce = DateUtil.createEventGC(event.get(imid).getEndTS());
          if(gce.before(gc)){
           imin = imid + 1;
          }else if(gce.after(gc)){
           imax = imid - 1;
          }else{
           return imid;
          }
          
        }
        return (imax < 0) ? 0 : imax;
    }
    public static String createQueryTS(GregorianCalendar ts) {
        return createQueryTS(DateUtil.makeTS(ts));
    }

    /**
     * Formats a ts for the db specified
     * @param ts a timestamp in the format yyyy-mm-dd hh24:mi:ss
     * @return a string wrapped with the correct DB TS information
     */
    public static String createQueryTS(String ts) {
        String ret = "";
        if(DB_CLASS == "org.sqlite.JDBC"){
         ret = "datetime(\'" + ts + "\')";
        }else{
         ret = "to_date('" + ts + "','yyyy-mm-dd hh24:mi:ss')";
        }
        return ret;
    }

    /**
     * Returns a list of CGMS readings
     * @param pid
     * @param start
     * @param end
     * @param table_with_where
     * @return list of CGMS readings between start and end inclusively 
     */
    public static List<Reading> getCGMS(int pid, GregorianCalendar start, GregorianCalendar end, String table_with_where) {
        List<Reading> points = new ArrayList<Reading>();
        try {
            String query = null;
            if (start == null || end == null) {
                query = "SELECT * FROM " + table_with_where + " PID = " + pid + " ORDER BY ts";
            } else {
                query = "SELECT * FROM " + table_with_where + " PID = " + pid + " AND TS >= " + DBUtil.createQueryTS(DateUtil.makeTS(start)) + " AND TS <= "
                        + DBUtil.createQueryTS(DateUtil.makeTS(end)) + " ORDER BY ts";
            }
            ResultSet rs = DBUtil.QueryDB(query);
            while (rs.next()) {
                points.add(new Reading(DateUtil.createGC(rs.getString("ts")), rs.getDouble("val")));
            }
            if(!conn.isClosed()){
             //conn.close();
            }
        } catch (SQLException ex) {
        } catch (NullPointerException ex){
          System.out.println("Null pointer exception: " + ex);
        }
        return points;
    }
    public static Reading nextCGMS(int pid, GregorianCalendar start, String table_with_where){
        Reading point = null;
        try {
            String query = null;
            if (start == null) {
                query = "SELECT * FROM " + table_with_where + " PID = " + pid + " AND ROWNUM=1 ORDER BY ts";
            } else {
                query = "SELECT * FROM " + table_with_where + " PID = " + pid + " AND TS > " + DBUtil.createQueryTS(DateUtil.makeTS(start)) + " AND ROWNUM=1 ORDER BY ts";
            }
            ResultSet rs = DBUtil.QueryDB(query);
            while (rs.next()) {
                point = new Reading(DateUtil.createGC(rs.getString("ts")), rs.getDouble("val"));
            }
            if(!conn.isClosed()){
             //conn.close();
            }
        } catch (SQLException ex) {
        } catch (NullPointerException ex){
          System.out.println("Null pointer exception: " + ex);
        }
        return point;
    }
    public static List<Reading> getFingersticks(int pid, GregorianCalendar start, GregorianCalendar end){
        List<Reading> points = new ArrayList<Reading>();
        try {
            
            String query = null;
            if (start == null || end == null) {
                query = "SELECT * FROM finger_stick WHERE PID = " + pid + " ORDER BY ts";
            } else {
                query = "SELECT * FROM finger_stick WHERE PID = " + pid + " AND TS >= " + DBUtil.createQueryTS(DateUtil.makeTS(start)) + " AND TS <= "
                        + DBUtil.createQueryTS(DateUtil.makeTS(end)) + " ORDER BY ts";
            }
            ResultSet rs = DBUtil.QueryDB(query);
            
            boolean finger_sticks_found = false;
            while (rs.next()) {
                points.add(new Reading(DateUtil.createGC(rs.getString("ts")), rs.getDouble("val")));
                finger_sticks_found = true;
            }
            if(!conn.isClosed() && finger_sticks_found){
             //conn.close();
            }
            rs.close();
        } catch (SQLException ex) {
        }
        return points;
    }

    /**
     * Queries the database, (all parameters are expected to be initiallized), prints an error message in case of error
     * @param conn
     * @param stmt
     * @param query
     * @return A result set from the query specified.
     */
    public static ResultSet QueryDB(String query) throws SQLException{
        ResultSet rset = null;
        if(conn == null){
         conn = DBUtil.createConnection();
        }
        if(stmt == null){
         stmt = conn.createStatement();
        }
        //        System.out.println("Query: " + query);
        if(conn.isClosed()){
         System.out.println("Conn is closed");
         return null;
        }
        try {
            rset = stmt.executeQuery(query);
        } catch (Exception e) {
            System.out.println("Problem executing query: " + query);
            System.out.println("Message     : " + e.getMessage());
            conn.clearWarnings();
            conn.close();
            conn = DBUtil.createConnection();
            stmt = conn.createStatement();
            if(conn.isClosed()){
             System.out.println("could not re-open connection");
             return null;
            }
            System.out.println("trying again");
            try{
              rset = stmt.executeQuery(query);
            }catch (Exception e2){
             System.out.println("Problem executing query: " + query);
             System.out.println("Message     : " + e2.getMessage());
             return null;
            }
            System.out.println("success");
        }
        return rset;
    }

    /**
     * Queries DB with no result set (i.e. insert delete)
     * For mass insertions, it is best to set the following options for SQLite (i.e. smoothing ALL data):
     *  PRAGMA synchronous=FALSE;
     *  PRAGMA cache_size=40000;
     *  PRAGMA count_changes=OFF;
     *  NOTE: This method should be set to recursively call its self when setting the above options.  This is because
     *    mass insertions with the above options turned on can cause DB lock errors.
     * This method may or may not recursively call its self in its current form.
     * @param conn
     * @param stmt
     * @param query
     */
    public static void QueryDBNoRS(String query) throws SQLException{
        ResultSet rset;
        if(conn == null){
         conn = DBUtil.createConnection();
        }
        if(stmt == null){
         stmt = conn.createStatement();
        }
        //        System.out.println("Query: " + query);
        try {
            stmt.execute(query);
        } catch (Exception e) {
            System.out.println("Problem executing query: " + query);
            System.out.println("Message     : " + e.getMessage());
            //QueryDBNoRS(conn, stmt, query);
        }
    }

    /**
     * Fills the daily schedule for the patient specified by PID
     * @param conn
     * @param PID
     * @return the daily schedule object for patient PID
     * @throws SQLException
     */
    public static DailySchedule fill_daily_schedule(Connection conn, int PID) throws SQLException {
        DailySchedule daily_schedule = new DailySchedule();
        Statement stmt = conn.createStatement();
        String query = "Select * from daily_work where DS_PID = " + PID;
        ResultSet rs = QueryDB(query);
        while (rs.next()) {
            daily_schedule.put_work(dowToNumber(rs.getString("ds_week_day")), DateUtil.createGC(rs.getString("goto_work")), DateUtil.createGC(rs.getString("return_work")));
        }
        stmt.close();
        rs.close();
        stmt = conn.createStatement();
        query = "SELECT * from daily_schedule where PID = " + PID;
        rs = QueryDB(query);
        while (rs.next()) {
            daily_schedule.put_daily_schedule(dowToNumber(rs.getString("week_day")), DateUtil.createGC(rs.getString("rise_time")),
                    DateUtil.createGC(rs.getString("bed_time")), DateUtil.createGC(rs.getString("breakfast")), DateUtil.createGC(rs.getString("lunch")),
                    DateUtil.createGC(rs.getString("dinner")));
        }
        stmt.close();
        rs.close();
        query = "SELECT * from daily_snack where ds_pid = " + PID;
        rs = QueryDB(query);
        while (rs.next()) {
            daily_schedule.put_daily_snack(dowToNumber(rs.getString("ds_week_day")), DateUtil.createGC(rs.getString("ts")));
        }
        stmt.close();
        rs.close();
        return daily_schedule;
    }

    /**
     *
     * @param dow the DAY_OF_WEEK value from the GregorianCalendar
     * @return int 1-7 representing sunday-saturday
     */
    public static int dowToNumber(String dow) {
        if (dow.toLowerCase().compareTo("sunday") == 0) {
            return 1;
        } else if (dow.toLowerCase().compareTo("monday") == 0) {
            return 2;
        } else if (dow.toLowerCase().compareTo("tuesday") == 0) {
            return 3;
        } else if (dow.toLowerCase().compareTo("wednesday") == 0) {
            return 4;
        } else if (dow.toLowerCase().compareTo("thursday") == 0) {
            return 5;
        } else if (dow.toLowerCase().compareTo("friday") == 0) {
            return 6;
        } else if (dow.toLowerCase().compareTo("saturday") == 0) {
            return 7;
        }
        return -1;
    }
}
