package util;

import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 * Static functions which are useful for date operations.
 * @author Matthew Wiley
 * @since 5-1-10
 */
public class DateUtil {

    /**
     * Creates an interval for a TS for the database type set by the constructor
     * @param ts Timestamp in string format of the interval to use
     * @param amount positive or negative number
     * @param amt_type valid options are second, minute, hour, day, month, year (s concatenated for oracle)
     * @return A string with the interval created wrapped with the correct DB syntax
     */
    public static String CreateIntervalForTS(String ts, int amount, String amt_type) {
        String interval = null;
        interval = "";
        interval = interval.concat("datetime(\'" + ts + "\', \'" + amount + " " + amt_type + "s\')");
        return interval;
    }

    /**
     * Copyies a gregoriancalendar
     * @param o calendar to copy
     * @return a copy of the calendar
     */
    public static GregorianCalendar copyGC(GregorianCalendar o) {
        GregorianCalendar ret = new GregorianCalendar(o.get(Calendar.YEAR), o.get(Calendar.MONTH), o.get(Calendar.DAY_OF_MONTH), o.get(Calendar.HOUR_OF_DAY), o.get(Calendar.MINUTE), o.get(Calendar.SECOND));
        return ret;
    }

    public static GregorianCalendar createGC(String date, String time) {
        String[] darr = date.split("-");
        String[] tarr = time.split(":");
        GregorianCalendar ret = new GregorianCalendar(Integer.parseInt(darr[0]), (Integer.parseInt(darr[1]) - 1), Integer.parseInt(darr[2]),
                Integer.parseInt(tarr[0]), Integer.parseInt(tarr[1]), (int)Double.parseDouble(tarr[2]));
        return ret;
    }
    public static GregorianCalendar createEventGC(String date) {
       if (date == null) {
            return null;
        }
        //System.out.println(date);
        String[] arr = date.split(" ");
        String[] darr = arr[0].split("-");
        String[] tarr = arr[1].split(":");
        GregorianCalendar ret = new GregorianCalendar(Integer.parseInt(darr[2]), Integer.parseInt(darr[1]) - 1, Integer.parseInt(darr[0]), Integer.parseInt(tarr[0]), Integer.parseInt(tarr[1]), (int)Double.parseDouble(tarr[2]));
        return ret;
    }

    /**
     * Creates a gregorian calendar from the yyyy-mm-dd hh24:mi:ss format
     * @param date
     * @return a gregorian calendar
     */
    public static GregorianCalendar createGC(String date) {
        if (date == null) {
            return null;
        }
        //System.out.println(date);
        String[] arr = date.split(" ");
        String[] darr = arr[0].split("-");
        String[] tarr = arr[1].split(":");
        GregorianCalendar ret = new GregorianCalendar(Integer.parseInt(darr[0]), Integer.parseInt(darr[1]) - 1, Integer.parseInt(darr[2]), Integer.parseInt(tarr[0]), Integer.parseInt(tarr[1]), (int)Double.parseDouble(tarr[2]));
        return ret;
    }

    public static double getDayDiff(GregorianCalendar start, GregorianCalendar end) {
        return (end.getTimeInMillis() - start.getTimeInMillis()) / ((double) 1000.0 * 60.0 * 60.0 * 24.0);
    }

    /**
     * Get hour different between start and end (start - end)
     * @param start
     * @param end
     * @return Hour Difference between start and end
     */
    public static double getHourDiff(GregorianCalendar start, GregorianCalendar end) {
        return (end.getTimeInMillis() - start.getTimeInMillis()) / ((double) 1000.0 * 60.0 * 60.0);
    }

    /**
     * Get minute different betweeen start and end (end - start)
     * @param start
     * @param end
     * @return Minute Difference between start and end
     */
    public static double getMinuteDiff(GregorianCalendar start, GregorianCalendar end) {
        return (end.getTimeInMillis() - start.getTimeInMillis()) / ((double) 1000.0 * 60.0);
    }

    /**
     * Get the second difference between start and end.
     * @param start
     * @param end
     * @return the difference in seconds
     */
    public static double getSecondDiff(GregorianCalendar start, GregorianCalendar end) {
        return (end.getTimeInMillis() - start.getTimeInMillis()) / ((double) 1000.0);
    }

    /**
     * makes a timestamp in (yyyy-mm-dd hh24:mi:ss) format from a GregorianCalendar
     * @param ts The gregoriancalendar to convert
     * @return a timestamp in the format mentioned above
     */
    public static String makeTS(GregorianCalendar ts) {
        String ret = "";
        ret = ret.concat(ts.get(Calendar.YEAR) + "-");
        if (ts.get(Calendar.MONTH) < 9) {
            ret = ret.concat("0");
        }
        ret = ret.concat((ts.get(Calendar.MONTH) + 1) + "-");
        if (ts.get(Calendar.DAY_OF_MONTH) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.DAY_OF_MONTH) + " ");
        if (ts.get(Calendar.HOUR_OF_DAY) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.HOUR_OF_DAY) + ":");
        if (ts.get(Calendar.MINUTE) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.MINUTE) + ":");
        if (ts.get(Calendar.SECOND) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.SECOND) + "");
        return ret;
    }

    /**
     * makes a ts with underscores format (yyyy_mm_dd_hh24_mi_ss)
     * @param ts the gregorian calendar to use
     * @return a timestamp with underscores
     */
    public static String makeTSWithUS(GregorianCalendar ts) {
        String ret = "";
        ret = ret.concat(ts.get(Calendar.YEAR) + "_");
        if (ts.get(Calendar.MONTH) < 9) {
            ret = ret.concat("0");
        }
        ret = ret.concat((ts.get(Calendar.MONTH) + 1) + "_");
        if (ts.get(Calendar.DAY_OF_MONTH) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.DAY_OF_MONTH) + "_");
        if (ts.get(Calendar.HOUR_OF_DAY) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.HOUR_OF_DAY) + "_");
        if (ts.get(Calendar.MINUTE) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.MINUTE) + "_");
        if (ts.get(Calendar.SECOND) < 10) {
            ret = ret.concat("0");
        }
        ret = ret.concat(ts.get(Calendar.SECOND) + "");
        return ret;
    }
}
