package util;

import java.util.Scanner;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Matthew Wiley
 * @since 5-1-10
 */
public class InputUtil {

    /**
     *
     * @param msg
     * @return a valid date matching the yyyy-mm-dd format
     */
    public static String getDate(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_DATE);
        return getInput(scan, opts);
    }

    /**
     *
     * @param msg
     * @return a valid time matching the hh24:mi format
     */
    public static String getTime(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_TIME);
        return getInput(scan, opts);
    }

    public static String getInput(Scanner scan, Pattern opts) {
        boolean valid = false;
        String tmp = new String();
        while (!valid) {
            System.out.print("Input: ");
            tmp = scan.nextLine();
            Matcher m = opts.matcher(tmp);
            if (m.find()) {
                valid = true;
            } else {
                System.out.println("***Invalid Input***");
            }
        }
        return tmp;
    }

    public static String getFileName(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_FILE_NAME);
        return getInput(scan, opts);
    }

    public static String getDirName(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_DIR_NAME);
        return getInput(scan, opts);
    }

    public static String getFileName(Scanner scan, boolean input) {
        if (input) {
            System.out.println("Enter the filename to read in");
        } else {
            System.out.println("Enter the filename to output to");
        }
        Pattern opts = Pattern.compile(VALID_FILE_NAME);
        return getInput(scan, opts);
    }

    /**
     *
     * @param msg
     * @return a valid PID (assumed to be 3 digits)
     */
    public static String getPID(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_PID);
        return getInput(scan, opts);
    }

    public static String getInteger(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_INT);
        return getInput(scan, opts);
    }

    public static String getHorizon(Scanner scan, String msg) {
        System.out.println(msg);
        Pattern opts = Pattern.compile(VALID_HORIZON);
        return getInput(scan, opts);
    }
    public static final String VALID_HORIZON = "^([1-9][0-9]|[1-9][0-9]|[1-9])$";
    public static final String VALID_DIR_NAME = "^([A-za-z0-9_]*)$";
    public static final String VALID_FILE_NAME = "^([A-Za-z0-9_]*\\.[A-Za-z0-9]*)$";
    public static final String VALID_INT = "^([1-9][0-9][0-9]|[1-9][0-9]|[1-9])$";
    public static final String VALID_PID = "^([1-9][0-9][0-9][0-9]|[1-9][0-9][0-9])$";
    public static final String VALID_DATE = "^((19|20)\\d\\d[-](0?[1-9]|1[012])[-](0?[1-9]|[12][0-9]|3[01]))$";
    public static final String VALID_TIME = "^((0?[0-9]|1[0-9]|2[0-4])[:](0?[0-9]|[1-5][0-9])[:](0?[0-9]|[1-5][0-9]))$";
    public static final String ONE_OR_TWO = "^([1-2])$";
    public static final String YES_OR_NO = "^([ynYN])$";
}
